VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "InfoAction"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

Public type_ As TBuildAction
Public args_ As Collection

Private Sub Class_Initialize()
  Set args_ = New Collection
End Sub

' ===== Properties =====
Public Property Get Description() As String
  Description = ActionType2Text(type_)
  If args_.Count > 0 Then _
    Description = Description & " " & args_(1)
  If args_.Count > 1 Then _
    Description = Description & " -> " & args_(2)
End Property

Public Property Get IsBuildOnly() As Boolean
  Select Case type_
    Case T_ACT_BUILD: IsBuildOnly = True
    Case T_ACT_COPY: IsBuildOnly = True
    Case T_ACT_SAVE_AS: IsBuildOnly = True
    Case T_ACT_RUN: IsBuildOnly = False
    Case T_ACT_INSTALL: IsBuildOnly = False
    Case T_ACT_TEMPLATE: IsBuildOnly = False
    Case Else: IsBuildOnly = False
  End Select
End Property

Public Property Get IsInstallOnly() As Boolean
  Select Case type_
    Case T_ACT_BUILD: IsInstallOnly = False
    Case T_ACT_COPY: IsInstallOnly = False
    Case T_ACT_SAVE_AS: IsInstallOnly = False
    Case T_ACT_RUN: IsInstallOnly = False
    Case T_ACT_INSTALL: IsInstallOnly = True
    Case T_ACT_TEMPLATE: IsInstallOnly = True
    Case Else: IsInstallOnly = False
  End Select
End Property

' ===== Actions =====
Public Function Execute(env As ItemActionEnvironment) As Boolean
  Select Case type_
    Case T_ACT_BUILD: Execute = ExecuteBuild(env)
    Case T_ACT_COPY: Execute = ExecuteCopy(env)
    Case T_ACT_SAVE_AS: Execute = ExecuteSaveAs(env)
    Case T_ACT_RUN: Execute = ExecuteRun(env)
    Case T_ACT_INSTALL: Execute = ExecuteInstall(env)
    Case T_ACT_TEMPLATE: Execute = ExecuteTemplate(env)
    Case Else: Execute = False
  End Select
End Function

' ======
Private Function ExecuteBuild(env As ItemActionEnvironment) As Boolean
  ExecuteBuild = False
  
  Dim sManifest$: sManifest = CPath(args_(1)).ToGlobal(env.sourceHome_).Text
  Dim iComponent As New InfoComponent
  If Not iComponent.Init(sManifest, env.sourceHome_, env.artifactHome_, env.productVersion_) Then
    Call env.reporter_.Log("Failed loading component manifest: " & sManifest)
    Exit Function
  End If
  
  Dim iData As IteratorComponent: Set iData = env.components_.Access(env.product_, iComponent.ItemName)
  If iData Is Nothing Then
    Call env.reporter_.Log("Failed accessing component: " & sManifest)
    Exit Function
  End If
  
  Dim testOut As IteratorTest: Set testOut = env.testDB_.OutputFor(env.product_, iComponent.ItemName)
  Dim buildRes As InfoBuild: Set buildRes = iComponent.Build(env.buildType_, env.reporter_, testOut)
  Call testOut.RemoveRow
  
  ExecuteBuild = buildRes.status_ = T_BS_OK
  Call iData.SyncBuildInfo(buildRes)
End Function

Private Function ExecuteCopy(env As ItemActionEnvironment) As Boolean
  Dim sSource$: sSource = CPath(args_(1)).ToGlobal(env.sourceHome_).Text
  Dim sDestination$
  If args_.Count < 2 Then
    sDestination = env.artifactHome_ & "\" & env.fso_.GetFileName(sSource)
  Else
    sDestination = CPath(args_(2)).ToGlobal(env.artifactHome_).Text
  End If
  ExecuteCopy = CopyFileOrFolder(sSource, sDestination, env.fso_)
End Function

Private Function ExecuteSaveAs(env As ItemActionEnvironment) As Boolean
  ExecuteSaveAs = False
  
  Dim iSource As API_Path: Set iSource = CPath(args_(1)).ToGlobal(env.artifactHome_)
  Dim iDestination As API_Path: Set iDestination = CPath(args_(2)).ToGlobal(env.artifactHome_)
  If iSource.Extension = iDestination.Extension Then
    Call CopyFileOrFolder(iSource.Text, iDestination.Text, env.fso_)
  Else
    Dim aWrap As Object: Set aWrap = AccessArtifact(iSource.Text, bReadOnly:=True)
    If aWrap Is Nothing Then
      Call env.reporter_.Log("Cannot access artifact: " & iSource.Text)
      Exit Function
    End If
    
    Call EnsureFolderExists(env.fso_.GetParentFolderName(iDestination.Text))
    Call aWrap.SaveAs(iDestination.Text)
    Call aWrap.ReleaseDocument(bSaveChanges:=True)
  End If
  
  ExecuteSaveAs = True
End Function

Private Function ExecuteRun(env As ItemActionEnvironment) As Boolean
  ExecuteRun = False
  
  Dim sExec$: sExec = args_(1)
  If VBA.InStr(1, sExec, ".") <> 0 Then _
    sExec = CPath(sExec).ToGlobal(IIf(env.isBuilding_, env.sourceHome_, env.artifactHome_)).Text
  
  Dim fso As New Scripting.FileSystemObject
  Dim sHome$: sHome = fso.GetParentFolderName(sExec)
  Dim isPowerShell As Boolean: isPowerShell = VBA.UCase(fso.GetExtensionName(sExec)) = "PS1"
  
  If isPowerShell Then _
    sExec = "PowerShell -NoProfile -ExecutionPolicy Bypass -Command ""& '" & sExec & "'"""
  If args_.Count > 1 Then _
    sExec = sExec & " " & args_(2)
    
  On Error GoTo EXECUTION_ERROR
  Dim iShell As New WshShell
  iShell.CurrentDirectory = sHome
  ExecuteRun = iShell.Run(sExec, waitOnReturn:=True) = 0
  On Error GoTo 0
  Exit Function
  
EXECUTION_ERROR:
  Call env.reporter_.Log("Failed to interpret shell command: " & sExec)
  On Error GoTo 0
End Function

Private Function ExecuteInstall(env As ItemActionEnvironment) As Boolean
  Dim iSource As API_Path: Set iSource = CPath(args_(1)).ToGlobal(env.artifactHome_)
  Dim sDestination$
  If args_.Count < 2 Then
    sDestination = env.installHome_ & "\" & iSource.FileName
  Else
    sDestination = CPath(args_(2)).ToGlobal(env.installHome_).Text
  End If
  ExecuteInstall = CopyFileOrFolder(iSource.Text, sDestination, env.fso_)
End Function

Private Function ExecuteTemplate(env As ItemActionEnvironment) As Boolean
  ExecuteTemplate = False
  
  Dim sSource$: sSource = env.artifactHome_ & "\" & args_(1)
  Dim sDestName$
  If args_.Count < 2 Then
    sDestName = env.fso_.GetFileName(sSource)
  Else
    sDestName = args_(2)
  End If
  
  Dim vPath As Variant
  For Each vPath In Globals.TemplatePaths
    Dim sDestination$: sDestination = CPath(sDestName).ToGlobal(CStr(vPath)).Text
    If Not CopyFileOrFolder(sSource, sDestination, env.fso_) Then
      Call env.reporter_.Log("Failed to install template: " & sDestination)
      Exit Function
    End If
  Next vPath
  
  ExecuteTemplate = True
End Function
