Attribute VB_Name = "Main"
Option Explicit

Public Sub RunEditVBAMake()
  Dim iProduct As IteratorProduct: Set iProduct = GetSelectedProduct
  If iProduct Is Nothing Then _
    Exit Sub
    
  Dim fso As New Scripting.FileSystemObject
  Dim sMake$: sMake = iProduct.Makefile
  If Not fso.FileExists(sMake) Then
    Call UserInteraction.ShowMessage(EM_MISSING_MAKEFILE, sMake)
    Exit Sub
  End If
  
  Dim oShell As New Shell32.Shell
  Call oShell.Open(iProduct.Makefile)
End Sub

Public Sub RunEditManifect()
  Dim iComp As IteratorComponent: Set iComp = GetSelectedComponent
  If iComp Is Nothing Then _
    Exit Sub
    
  Dim fso As New Scripting.FileSystemObject
  Dim sManifest$: sManifest = iComp.ManifestFile
  If Not fso.FileExists(sManifest) Then
    Call UserInteraction.ShowMessage(EM_MISSING_MANIFEST, sManifest)
    Exit Sub
  End If
  
  Dim oShell As New Shell32.Shell
  Call oShell.Open(sManifest)
End Sub

Public Sub RunOpenSrc()
  Dim iModel As InfoProduct: Set iModel = GetProductModel(GetSelectedProduct)
  If iModel Is Nothing Then _
    Exit Sub
  
  Dim oShell As New Shell32.Shell
  Call oShell.Open(iModel.SourceHome)
End Sub

Public Sub RunOpenArtifact()
  Dim iModel As InfoProduct: Set iModel = GetProductModel(GetSelectedProduct)
  If iModel Is Nothing Then _
    Exit Sub
  
  Dim oShell As New Shell32.Shell
  Call oShell.Open(iModel.ArtifactHome)
End Sub

Public Sub RunTestVBAMake()
  Dim iModel As InfoProduct: Set iModel = GetProductModel(GetSelectedProduct)
  If iModel Is Nothing Then _
    Exit Sub

  Dim oErrors As Collection: Set oErrors = iModel.ValidateMakefile()
  If oErrors Is Nothing Then
    Call UserInteraction.ShowMessage(IM_MAKEFILE_FAILED, "")
  ElseIf oErrors.Count = 0 Then
    Call UserInteraction.ShowMessage(IM_MAKEFILE_OK)
  Else
    Call UserInteraction.ShowMessage(IM_MAKEFILE_FAILED, CollectionToLines(oErrors))
  End If
End Sub

Public Sub RunBuildProduct()
  Dim theTimer As New API_Timer: Call theTimer.Start
  Dim iProduct As IteratorProduct: Set iProduct = GetSelectedProduct
  Dim iModel As InfoProduct: Set iModel = GetProductModel(iProduct)
  If iModel Is Nothing Then _
    Exit Sub
  
  Call ClearTests
  If BuildProduct(Globals.build_, iProduct, iModel, Globals.Logger) Then
    Call UserInteraction.ShowMessage(IM_BUILD_OK, theTimer.TimeStr)
  Else
    Call UserInteraction.ShowMessage(IM_BUILD_FAILED)
  End If
End Sub

Public Sub RunInstallProduct()
  Dim theTimer As New API_Timer: Call theTimer.Start
  Dim iProduct As IteratorProduct: Set iProduct = GetSelectedProduct
  Dim iModel As InfoProduct: Set iModel = GetProductModel(iProduct)
  If iModel Is Nothing Then _
    Exit Sub
  
  If InstallProduct(Globals.build_, iProduct, iModel, Globals.Logger) Then
    Call UserInteraction.ShowMessage(IM_INSTALL_OK, theTimer.TimeStr)
  Else
    Call UserInteraction.ShowMessage(IM_INSTALL_FAILED)
  End If
End Sub

Public Sub RunAddProduct()
  Dim sFolder$: sFolder = UserInteraction.PromptFolder(Globals.SourceHome & "\")
  If sFolder = vbNullString Then _
    Exit Sub
  
  Dim sFile$: sFile = sFolder & "\" & MAKEFILE_NAME
  Dim fso As New Scripting.FileSystemObject
  If Not fso.FileExists(sFile) Then
    Call UserInteraction.ShowMessage(EM_MAKEFILE_MISSING, sFile)
    Exit Sub
  End If
  
  Dim iMake As New InfoMakefile
  If Not iMake.Init(sFolder & "\" & MAKEFILE_NAME) Then
    Call UserInteraction.ShowMessage(EM_MAKEFILE_LOADING_FAILED)
    Exit Sub
  End If
  
  Dim dbProducts As DB_Products: Set dbProducts = AccessProducts
  If Not dbProducts.Insert(iMake) Then
    Call UserInteraction.ShowMessage(EM_PRODUCT_ALREADY_EXISTS, iMake.ProductName)
  Else
    Call UserInteraction.ShowMessage(IM_PRODUCT_ADDED, iMake.ProductName)
  End If
End Sub

Public Sub RunArchiveVersion()
  Dim iProduct As IteratorProduct: Set iProduct = GetSelectedProduct
  Dim iModel As InfoProduct: Set iModel = GetProductModel(iProduct)
  If iModel Is Nothing Then _
    Exit Sub
  
  Dim sArchive$: sArchive = ArchiveNameFor(iModel, Now)
  sArchive = CreateVersionArchive(Globals.build_, sArchive, iModel, Globals.Logger)
  If sArchive = vbNullString Then
    Call UserInteraction.ShowMessage(IM_BUILD_FAILED)
    Exit Sub
  End If
  
  Dim fso As New Scripting.FileSystemObject
  Dim oShell As New Shell32.Shell
  Call oShell.Open(fso.GetParentFolderName(sArchive))
End Sub

Public Sub RunTestManifest()
  Dim iComp As IteratorComponent: Set iComp = GetSelectedComponent
  Dim iModel As InfoComponent: Set iModel = GetComponentModel(iComp)
  If iModel Is Nothing Then _
    Exit Sub
    
  Dim oErrors As Collection: Set oErrors = iModel.ValidateManifest(Globals.build_)
  If oErrors Is Nothing Then
    Call iComp.ResetBuildStatus
    Call UserInteraction.ShowMessage(IM_MANIFEST_FAILED, "")
  ElseIf oErrors.Count = 0 Then
    Call UserInteraction.ShowMessage(IM_MANIFEST_OK)
  Else
    Call iComp.ResetBuildStatus
    Call UserInteraction.ShowMessage(IM_MANIFEST_FAILED, CollectionToLines(oErrors))
  End If
End Sub

Public Sub RunOpenComponent()
  Dim iComp As IteratorComponent: Set iComp = GetSelectedComponent
  Dim iModel As InfoComponent: Set iModel = GetComponentModel(iComp)
  If iModel Is Nothing Then _
    Exit Sub
  
  Dim iDoc As Object: Set iDoc = iModel.OpenFileInNewProcess(bReadOnly:=False)
  If Not iDoc Is Nothing Then _
    If Not TypeOf iDoc Is Visio.Document Then _
      Call iDoc.Activate
End Sub

Public Sub RunOpenComponentReadOnly()
  Dim iComp As IteratorComponent: Set iComp = GetSelectedComponent
  Dim iModel As InfoComponent: Set iModel = GetComponentModel(iComp)
  If iModel Is Nothing Then _
    Exit Sub
    
  Dim iDoc As Object: Set iDoc = iModel.OpenFileInNewProcess(bReadOnly:=True)
  If Not iDoc Is Nothing Then _
    Call iDoc.Activate
End Sub

Public Sub RunBuildComponent()
  Dim theTimer As New API_Timer: Call theTimer.Start
  Dim iComp As IteratorComponent: Set iComp = GetSelectedComponent
  Dim iModel As InfoComponent: Set iModel = GetComponentModel(iComp)
  If iModel Is Nothing Then _
    Exit Sub
  If iModel.ItemName = ThisWorkbook.Name Then
    Call UserInteraction.ShowMessage(EM_SELF_MODIFICATION)
    Exit Sub
  End If
  
  Call ClearTests
  
  Dim buildRes As InfoBuild
  Set buildRes = iModel.Build(Globals.build_, Globals.Logger, AccessTests.OutputFor(iComp.Product, iComp.ItemName))
  Call iComp.SyncBuildInfo(buildRes)
  If buildRes.status_ = T_BS_OK Then
    Call UserInteraction.ShowMessage(IM_BUILD_OK, theTimer.TimeStr)
  Else
    Call UserInteraction.ShowMessage(IM_BUILD_FAILED)
  End If
End Sub

Public Sub RunTestComponent()
  Dim iComp As IteratorComponent: Set iComp = GetSelectedComponent
  Dim iModel As InfoComponent: Set iModel = GetComponentModel(iComp)
  If iModel Is Nothing Then _
    Exit Sub
  If iModel.ItemName = ThisWorkbook.Name Then
    Call UserInteraction.ShowMessage(EM_SELF_MODIFICATION)
    Exit Sub
  End If
  
  Call ClearTests
  
  Dim theTimer As New API_Timer: Call theTimer.Start
  
  Dim iOutput As IteratorTest: Set iOutput = AccessTests.OutputFor(iComp.Product, iComp.ItemName)
  Dim testRes As CDS_InfoTests: Set testRes = iModel.Test(Globals.Logger, iOutput)
  If testRes Is Nothing Then
    Call UserInteraction.ShowMessage(EM_TESTS_FAILED)
    Exit Sub
  End If
  Call iComp.SyncTestInfo(testRes)
  Call iOutput.RemoveRow
  
  If testRes.failed_ <> 0 Then _
    iComp.BuildStatus = T_BS_FAILED
  If testRes.count_ = 0 Then
    Call UserInteraction.ShowMessage(IM_TESTS_NOT_FOUND)
  Else
    Call UserInteraction.ShowMessage(IM_TESTS_COMPLETE, testRes.count_, testRes.success_, testRes.failed_, theTimer.TimeStr)
  End If
End Sub

Public Sub RunExportCode()
  Dim iComp As IteratorComponent: Set iComp = GetSelectedComponent
  Dim iModel As InfoComponent: Set iModel = GetComponentModel(iComp)
  If iModel Is Nothing Then _
    Exit Sub
  
  Call iComp.ResetBuildStatus
  If Not iModel.ExportCode() Then _
    Call UserInteraction.ShowMessage(EM_EXPORT_FAILED)
End Sub

Public Sub RunExportShared()
  Dim iComp As IteratorComponent: Set iComp = GetSelectedComponent
  Dim iModel As InfoComponent: Set iModel = GetComponentModel(iComp)
  If iModel Is Nothing Then _
    Exit Sub
  
  Call iComp.ResetBuildStatus
  If Not iModel.ExportShared() Then _
    Call UserInteraction.ShowMessage(EM_EXPORT_FAILED)
End Sub

Public Sub RunReloadCode()
  Dim iComp As IteratorComponent: Set iComp = GetSelectedComponent
  Dim iModel As InfoComponent: Set iModel = GetComponentModel(iComp)
  If iModel Is Nothing Then _
    Exit Sub
  If iModel.ItemName = ThisWorkbook.Name Then
    Call UserInteraction.ShowMessage(EM_SELF_MODIFICATION)
    Exit Sub
  End If
  
  Call iComp.ResetBuildStatus
  If Not iModel.ReloadCode() Then
    Call iComp.SetBuildStatus(T_BS_FAILED)
    Call UserInteraction.ShowMessage(EM_IMPORT_FAILED)
  End If
End Sub

Public Sub RunReloadShared()
  Dim iComp As IteratorComponent: Set iComp = GetSelectedComponent
  Dim iModel As InfoComponent: Set iModel = GetComponentModel(iComp)
  If iModel Is Nothing Then _
    Exit Sub
  If iModel.ItemName = ThisWorkbook.Name Then
    Call UserInteraction.ShowMessage(EM_SELF_MODIFICATION)
    Exit Sub
  End If
  
  Call iComp.ResetBuildStatus
  If Not iModel.ReloadShared() Then
    Call iComp.SetBuildStatus(T_BS_FAILED)
    Call UserInteraction.ShowMessage(EM_IMPORT_FAILED)
  End If
End Sub

Public Sub RunUpdateTarget()
  Dim iModel As InfoComponent: Set iModel = GetComponentModel(GetSelectedComponent)
  If iModel Is Nothing Then _
    Exit Sub

  Dim sFile$: sFile = UserInteraction.PromptFile(ThisWorkbook.Path & "\")
  If sFile = vbNullString Then _
    Exit Sub
    
  Dim fso As New Scripting.FileSystemObject
  If Not fso.FileExists(sFile) Then
    Call UserInteraction.ShowMessage(EM_MISSING_FILE, sFile)
    Exit Sub
  End If
  
  If UpdateCodeIn(iModel, sFile) Then
    Call UserInteraction.ShowMessage(IM_UPDATE_OK)
  Else
    Call UserInteraction.ShowMessage(IM_UPDATE_FAILED)
  End If
End Sub

Public Sub RunTargetClearCode()
  Dim sFile$: sFile = UserInteraction.PromptFile(ThisWorkbook.Path & "\")
  If sFile = vbNullString Then _
    Exit Sub
    
  Dim fso As New Scripting.FileSystemObject
  If Not fso.FileExists(sFile) Then
    Call UserInteraction.ShowMessage(EM_MISSING_FILE, sFile)
    Exit Sub
  End If
  
  If Not UserInteraction.AskQuestion(QM_CODE_DELETE_CONFIRM) Then _
    Exit Sub
    
  Dim wrap As Object: Set wrap = AccessArtifact(sFile, bReadOnly:=False)
  If wrap Is Nothing Then _
    Exit Sub
  
  Dim proj As New API_Project: Call proj.Init(wrap.Document.VBProject, "")
  Call proj.RemoveAll
  
  Call wrap.ReleaseDocument(bSaveChanges:=True)
End Sub

Public Sub RunTargetUpdateShared()
  Dim sFile$: sFile = UserInteraction.PromptFile(ThisWorkbook.Path & "\")
  If sFile = vbNullString Then _
    Exit Sub
    
  Dim fso As New Scripting.FileSystemObject
  If Not fso.FileExists(sFile) Then
    Call UserInteraction.ShowMessage(EM_MISSING_FILE, sFile)
    Exit Sub
  End If
  
  Dim wrap As Object: Set wrap = AccessArtifact(sFile, bReadOnly:=False)
  If wrap Is Nothing Then _
    Exit Sub
  
  Dim iUpdated As Collection: Set iUpdated = UpdateSharedIn(Globals.SharedHome, wrap.Document.VBProject)
  Call wrap.ReleaseDocument(bSaveChanges:=True)
  
  If iUpdated Is Nothing Then
    Call UserInteraction.ShowMessage(IM_UPDATE_SHARED_FAILED, iUpdated.Count)
  Else
    Call UserInteraction.ShowMessage(IM_UPDATE_SHARED_OK, iUpdated.Count, CollectionToLines(iUpdated))
  End If
End Sub

Public Sub RunOpenLog()
  Dim oShell As New Shell32.Shell
  Call oShell.Open(Globals.Logger.LogFileName)
End Sub

Public Sub RunOpenConfig()
  Dim oShell As New Shell32.Shell
  Call oShell.Open(VBA.Environ$("USERPROFILE") & "\.concept\concept-options.json")
End Sub

Public Sub RunOpenConcept()
  Dim oShell As New Shell32.Shell
  Call oShell.Open(VBA.Environ$("USERPROFILE") & "\.concept")
End Sub

Public Sub RunOpenVBCommons()
  Dim oShell As New Shell32.Shell
  Call oShell.Open(Globals.SharedHome)
End Sub

Public Sub RunOpenAppData()
  Dim oShell As New Shell32.Shell
  Call oShell.Open(VBA.Environ$("APPDATA"))
End Sub

Public Sub RunOpenUserProfile()
  Dim oShell As New Shell32.Shell
  Call oShell.Open(VBA.Environ$("USERPROFILE"))
End Sub

Public Sub RunOpenCIHT()
  Dim oShell As New Shell32.Shell
  Call oShell.Open(SERVER_PATH_CIHT)
End Sub

Public Sub RunOpenInstall()
  Dim oShell As New Shell32.Shell
  Call oShell.Open(SERVER_PATH_AUTOMATION)
End Sub

Public Sub RunListRefs()
  Dim iData As IteratorComponent: Set iData = GetSelectedComponent
  If iData Is Nothing Then _
    Exit Sub
    
  Dim appWrap As Object: Set appWrap = AccessArtifact(iData.ArtifactFile, bReadOnly:=True)
  If appWrap Is Nothing Then _
    Exit Sub
  
  Dim xlOut As New API_XLWrapper: Call xlOut.SetApplication(ThisWorkbook.Application)
  Call ScanRefsFrom(appWrap.Document.VBProject, xlOut.NewDocument.Worksheets(1))
  Call appWrap.ReleaseDocument
  Call xlOut.Document.Activate
End Sub

Public Sub RunListModules()
  Dim iData As IteratorComponent: Set iData = GetSelectedComponent
  If iData Is Nothing Then _
    Exit Sub
    
  Dim appWrap As Object: Set appWrap = AccessArtifact(iData.ArtifactFile, bReadOnly:=True)
  If appWrap Is Nothing Then _
    Exit Sub
    
  Dim xlOut As New API_XLWrapper: Call xlOut.SetApplication(ThisWorkbook.Application)
  Call ScanModules(appWrap.Document.VBProject, xlOut.NewDocument.Worksheets(1))
  Call appWrap.ReleaseDocument
  Call xlOut.Document.Activate
End Sub

Public Sub RunListShared()
  Call ClearShared
  
  Dim dbShared As DB_SharedModules: Set dbShared = AccessShared
  Dim iOut As IteratorSharedModule: Set iOut = dbShared.INew
  
  Dim oErrors As Collection: Set oErrors = ScanShared(iOut, Globals.SharedHome, AccessComponents)
  Call ThisWorkbook.Sheets(SHEET_SHARED).Activate
  
  If oErrors.Count <> 0 Then _
    Call UserInteraction.ShowMessage(IM_SHARED_INVALID, CollectionToLines(oErrors))
End Sub

Public Sub RunReloadAllShared()
  Dim iData As IteratorComponent: Set iData = AccessComponents.IBegin
  Dim oErrors As New Collection
  Do While Not iData.IsDone
    Dim iModel As InfoComponent: Set iModel = iData.Model
    If iModel Is Nothing Then
      Call oErrors.Add(iData.ItemName)
    ElseIf Not iModel.ReloadShared Then
      Call iData.SetBuildStatus(T_BS_FAILED)
      Call oErrors.Add(iData.ItemName)
    End If
    Call iData.ResetBuildStatus
    Call iData.Increment
  Loop

  If oErrors.Count = 0 Then
    Call UserInteraction.ShowMessage(IM_RELOAD_OK)
  Else
    Call UserInteraction.ShowMessage(IM_RELOAD_FAILED, CollectionToLines(oErrors))
  End If
End Sub

Public Sub RunCompileAll()
  Dim iData As IteratorComponent: Set iData = AccessComponents.IBegin
  Dim oErrors As New Collection
  Do While Not iData.IsDone
    Dim iModel As InfoComponent: Set iModel = iData.Model
    If iModel Is Nothing Then
      Call oErrors.Add(iData.ItemName)
    ElseIf Not iModel.Compile Then
      Call oErrors.Add(iData.ItemName)
    End If
    Call iData.Increment
  Loop
  
  If oErrors.Count = 0 Then
    Call UserInteraction.ShowMessage(IM_COMPILATION_OK)
  Else
    Call UserInteraction.ShowMessage(IM_COMPILATION_FAILED, CollectionToLines(oErrors))
  End If
End Sub

Public Sub RunAllTests()
  Call ClearTests
  
  Dim theTimer As New API_Timer: Call theTimer.Start
  Dim reporter As API_Logger: Set reporter = Globals.Logger
  Dim totalRes As New CDS_InfoTests
  
  Dim iData As IteratorComponent: Set iData = AccessComponents.IBegin
  Do While Not iData.IsDone
    Dim iModel As InfoComponent: Set iModel = iData.Model
    If iModel Is Nothing Then _
      GoTo NEXT_COMPONENT
    
    Dim iOutput As IteratorTest: Set iOutput = AccessTests.OutputFor(iData.Product, iData.ItemName)
    Dim localRes As CDS_InfoTests: Set localRes = iModel.Test(reporter, iOutput)
    If localRes Is Nothing Then
      iData.BuildStatus = T_BS_FAILED
      GoTo NEXT_COMPONENT
    End If
    If localRes.failed_ <> 0 Then _
      iData.BuildStatus = T_BS_FAILED
    
    Call iData.SyncTestInfo(localRes)
    Call totalRes.MergeStats(localRes)
    Call iOutput.RemoveRow
    
NEXT_COMPONENT:
    Call iData.Increment
  Loop
  
  Call UserInteraction.ShowMessage(IM_TESTS_COMPLETE, totalRes.count_, totalRes.success_, totalRes.failed_, theTimer.TimeStr)
End Sub

Public Sub RunBuildAll()
  Call ClearTests
  
  Dim theTimer As New API_Timer: Call theTimer.Start
  Dim bType As TBuildType: bType = Globals.build_
  Dim reporter As API_Logger: Set reporter = Globals.Logger
  Dim iProduct As IteratorProduct: Set iProduct = AccessProducts.IBegin
  Dim nCount&: nCount = 0
  Dim nSuccess&: nSuccess = 0
  Do While Not iProduct.IsDone
    nCount = nCount + 1
    Dim iModel As InfoProduct: Set iModel = iProduct.Model(AccessComponents)
    If iModel Is Nothing Then
      Call iProduct.SetBuildStatus(T_BS_FAILED)
      GoTo NEXT_PRODUCT
    End If
    
    If BuildProduct(bType, iProduct, iModel, reporter) Then _
      nSuccess = nSuccess + 1

NEXT_PRODUCT:
    Call iProduct.Increment
  Loop
  
  Call UserInteraction.ShowMessage(IM_BATCH_BUILD, nSuccess, nCount, theTimer.TimeStr)
End Sub

Public Sub RunInstallAll()
  Call ClearTests
  
  Dim theTimer As New API_Timer: Call theTimer.Start
  Dim bType As TBuildType: bType = Globals.build_
  Dim reporter As API_Logger: Set reporter = Globals.Logger
  Dim iProduct As IteratorProduct: Set iProduct = AccessProducts.IBegin
  Dim nCount&: nCount = 0
  Dim nSuccess&: nSuccess = 0
  Do While Not iProduct.IsDone
    nCount = nCount + 1
    Dim iModel As InfoProduct: Set iModel = iProduct.Model(AccessComponents)
    If iModel Is Nothing Then
      Call iProduct.SetBuildStatus(T_BS_FAILED)
      GoTo NEXT_PRODUCT
    End If
    
    If InstallProduct(bType, iProduct, iModel, reporter) Then _
      nSuccess = nSuccess + 1

NEXT_PRODUCT:
    Call iProduct.Increment
  Loop
  
  Call UserInteraction.ShowMessage(IM_BATCH_INSTALL, nSuccess, nCount, theTimer.TimeStr)
End Sub

Public Sub RunUpdateSkeleton()
  Dim iModel As InfoComponent: Set iModel = GetComponentModel(GetSelectedComponent)
  If iModel Is Nothing Then _
    Exit Sub

  If Not iModel.UpdateSkeleton() Then _
    Call UserInteraction.ShowMessage(EM_SKELETON_FAILED)
End Sub

Public Sub RunScanFolder()
  Dim sFolder$: sFolder = UserInteraction.PromptFolder(Globals.SourceHome & "\")
  If sFolder = vbNullString Then _
    Exit Sub
  
  Dim nCount&: nCount = ScanProducts(sFolder)
  Call UserInteraction.ShowMessage(IM_SCAN_OK, nCount)
End Sub

' ======
Private Function GetSelectedComponent() As IteratorComponent
  Dim sel As Excel.Range: Set sel = Excel.Selection
  If ThisWorkbook.ActiveSheet.Name <> SHEET_COMPONENTS _
      Or sel.Parent.Parent.Name <> ThisWorkbook.Name _
      Or sel.Rows.Count <> 1 Then
    Call UserInteraction.ShowMessage(EM_INVALID_SELECTION)
    Exit Function
  End If
  
  Dim nRow&: nRow = sel.Cells(1, 1).Row
  If nRow < FIRST_ROW Then
    Call UserInteraction.ShowMessage(EM_INVALID_SELECTION)
    Exit Function
  End If
  
  Dim res As New IteratorComponent: Call res.Init(ThisWorkbook.Sheets(SHEET_COMPONENTS), nRow)
  If res.IsDone() Then
    Call UserInteraction.ShowMessage(EM_INVALID_SELECTION)
    Exit Function
  End If
  
  Set GetSelectedComponent = res
End Function

Private Function GetComponentModel(target As IteratorComponent) As InfoComponent
  If target Is Nothing Then _
    Exit Function
  Dim iModel As InfoComponent: Set iModel = target.Model
  If iModel Is Nothing Then
    Call UserInteraction.ShowMessage(EM_INVALID_COMPONENT)
    Exit Function
  End If
  Set GetComponentModel = iModel
End Function

Private Function GetSelectedProduct() As IteratorProduct
  Dim sel As Excel.Range: Set sel = Excel.Selection
  If sel.Parent.Parent.Name <> ThisWorkbook.Name _
      Or sel.Rows.Count <> 1 Then
    Call UserInteraction.ShowMessage(EM_INVALID_SELECTION)
    Exit Function
  End If
  
  Dim nRow&: nRow = sel.Cells(1, 1).Row
  If nRow < FIRST_ROW Then
    Call UserInteraction.ShowMessage(EM_INVALID_SELECTION)
    Exit Function
  End If
  
  Dim iTable As Excel.ListObject: Set iTable = sel.ListObject
  If iTable Is Nothing Then _
    GoTo MISSING_PRODUCT
  Dim iCell As Excel.Range: Set iCell = iTable.HeaderRowRange.Cells.Find("ProductName")
  If iCell Is Nothing Then _
    GoTo MISSING_PRODUCT
    
  Dim sName$: sName = sel.Parent.Cells(nRow, iCell.Column)
  Dim projects As New DB_Products: Set projects = AccessProducts
  If projects.Contains(sName) Then
    Set GetSelectedProduct = projects.Access(sName)
  Else
MISSING_PRODUCT:
    Call UserInteraction.ShowMessage(EM_MISSING_PRODUCT, sName)
  End If
End Function

Private Function GetProductModel(target As IteratorProduct) As InfoProduct
  If target Is Nothing Then _
    Exit Function
  Dim iModel As InfoProduct: Set iModel = target.Model(AccessComponents)
  If iModel Is Nothing Then
    Call UserInteraction.ShowMessage(EM_INVALID_PRODUCT)
    Exit Function
  End If
  Set GetProductModel = iModel
End Function
