Attribute VB_Name = "MainImpl"
Option Private Module
Option Explicit

Public Function ClearTests()
  Call ThisWorkbook.Sheets(SHEET_TESTS).UsedRange.Offset(1, 0).ClearContents
End Function

Public Function ClearShared()
  Call ThisWorkbook.Sheets(SHEET_SHARED).UsedRange.Offset(1, 0).ClearContents
End Function

Public Function CPath(sPath$) As API_Path
  Set CPath = New API_Path
  Call CPath.FromString(sPath)
End Function

Public Function ScanRefsFrom(target As VBIDE.VBProject, iOut As Excel.Worksheet)
  iOut.Cells(1, S_R_NAME) = "Name"
  iOut.Cells(1, S_R_TYPE) = "Type"
  iOut.Cells(1, S_R_FILE) = "File"
  iOut.Cells(1, S_R_GUID) = "GUID"
  iOut.Cells(1, S_R_BROKEN) = "IsBroken"
  iOut.Cells(1, S_R_MAJOR) = "Major"
  iOut.Cells(1, S_R_MINOR) = "Minor"
  iOut.Cells(1, S_R_DESCRIPTION) = "Description"
  
  Dim rowOut&: rowOut = 2
  Dim aRef As VBIDE.Reference
  For Each aRef In target.References
    iOut.Cells(rowOut, S_R_NAME) = aRef.Name
    iOut.Cells(rowOut, S_R_TYPE) = aRef.Type
    iOut.Cells(rowOut, S_R_FILE) = aRef.FullPath
    iOut.Cells(rowOut, S_R_GUID) = aRef.GUID
    iOut.Cells(rowOut, S_R_BROKEN) = IIf(aRef.IsBroken, 1, 0)
    iOut.Cells(rowOut, S_R_MAJOR) = aRef.Major
    iOut.Cells(rowOut, S_R_MINOR) = aRef.Minor
    iOut.Cells(rowOut, S_R_DESCRIPTION) = aRef.Description
    rowOut = rowOut + 1
  Next aRef
End Function

Public Function ScanModules(target As VBIDE.VBProject, iOut As Excel.Worksheet)
  Dim fso As New Scripting.FileSystemObject
  
  iOut.Cells(1, S_M_NAME) = "Name"
  iOut.Cells(1, S_M_TYPE) = "Type"
  iOut.Cells(1, S_M_SHARED) = "Shared"
  iOut.Cells(1, S_M_LINES_TOTAL) = "LoC"
  iOut.Cells(1, S_M_LINES_DECLARATIONS) = "LoD"
  iOut.Cells(1, S_M_COUNT_API) = "API"
  
  Dim rowOut&: rowOut = 2
  Dim aComp As VBIDE.VBComponent
  For Each aComp In target.VBComponents
    If aComp.CodeModule.CountOfLines >= LOC_MINIMUM Then
      iOut.Cells(rowOut, S_M_NAME) = aComp.Name
      iOut.Cells(rowOut, S_M_TYPE) = aComp.Type
      iOut.Cells(rowOut, S_M_SHARED) = GetSharedPrefix(aComp)
      iOut.Cells(rowOut, S_M_LINES_TOTAL) = aComp.CodeModule.CountOfLines
      iOut.Cells(rowOut, S_M_LINES_DECLARATIONS) = aComp.CodeModule.CountOfDeclarationLines
      iOut.Cells(rowOut, S_M_COUNT_API) = Dev_CountPublicAPI(aComp.CodeModule)
      rowOut = rowOut + 1
    End If
  Next aComp
End Function

Public Function ScanShared(iOut As IteratorSharedModule, sSource$, dbComponents As DB_Components) As Collection
  Dim fso As New Scripting.FileSystemObject
  If Not fso.FolderExists(sSource) Then _
    Exit Function
    
  Dim xlWrap As New API_XLWrapper: Call xlWrap.SetApplication(ThisWorkbook.Application)
  If xlWrap.NewDocument() Is Nothing Then _
    Exit Function
    
  Dim oInserter As New CB_AddModule: Call oInserter.Init(iOut, xlWrap.Document.VBProject, fso.GetFolder(sSource).Path)
  Call ForEachFileRecursive(sSource, oInserter, "TryAddModule")
  Call xlWrap.ReleaseDocument
  
  Set ScanShared = UpdateSharedUsage(iOut, dbComponents)
End Function

Public Function UpdateSharedIn(sShared$, target As VBIDE.VBProject) As Collection
  Dim iShared As Scripting.Dictionary: Set iShared = GetAllSharedModules(sShared)
  Dim iUpdated As New Collection
  
  Dim iProject As New API_Project: Call iProject.Init(target)
  
  Dim aComponent As VBComponent
  For Each aComponent In target.VBComponents
    Dim sName$: sName = aComponent.Name
    If iShared.Exists(sName) Then _
      If iProject.ReloadFrom(iShared(sName)) Then _
        Call iUpdated.Add(sName)
  Next aComponent
  
  If CompileVBProject(target) Then _
    Set UpdateSharedIn = iUpdated
End Function

Public Function BuildProduct(bType As TBuildType, iProduct As IteratorProduct, iModel As InfoProduct, reporter As API_Logger) As Boolean
  BuildProduct = iModel.Build(bType, reporter, AccessComponents, AccessTests)
  Call iProduct.SetBuildStatus(IIf(BuildProduct, T_BS_OK, T_BS_FAILED))
End Function

Public Function InstallProduct(bType As TBuildType, iProduct As IteratorProduct, iModel As InfoProduct, reporter As API_Logger) As Boolean
  If iProduct.Status <> T_BS_OK Then
    Call reporter.Log("The project wasnt built... Rebuilding")
    If Not BuildProduct(bType, iProduct, iModel, reporter) Then
      InstallProduct = False
      Exit Function
    End If
  End If
  
  InstallProduct = iModel.Install(bType, reporter, AccessComponents, iModel.InstallHome)
  If InstallProduct Then _
    iProduct.InstallDate = Now
End Function

Public Function UpdateCodeIn(iModel As InfoComponent, sTargetFile$) As Boolean
  Call iModel.manifest_.properties_.Remove(MANIFEST_PROPS_ARTIFACT)
  Call iModel.manifest_.properties_.Add(MANIFEST_PROPS_ARTIFACT, sTargetFile)
  UpdateCodeIn = iModel.ReloadAll
End Function

Public Function IsBasicReference(target As VBIDE.Reference, proj As VBIDE.VBProject) As Boolean
  If VBA.InStr(1, Globals.DefaultRefs, target.Name) <> 0 Then
    IsBasicReference = True
    Exit Function
  End If
  
  Dim fso As New FileSystemObject
  Dim tAppl As TApplication: tAppl = ApplicationFromExtension(fso.GetExtensionName(proj.FileName))
  Select Case tAppl
    Case T_APP_WORD: IsBasicReference = target.Name = "Word"
    Case T_APP_EXCEL: IsBasicReference = target.Name = "Excel"
    Case T_APP_VISIO: IsBasicReference = target.Name = "Visio"
    Case Else: IsBasicReference = False
  End Select
End Function

Public Function CreateVersionArchive(bType As TBuildType, sArchiveFolder$, target As InfoProduct, theLog As API_Logger) As String
  Call ClearTests
  
  Call theLog.Log("Preparing version files... Rebuilding project")
  If Not target.Build(bType, theLog, AccessComponents, AccessTests, sArchiveFolder) Then _
    GoTo SAFE_EXIT
    
  Call theLog.Log("Creating archive")
  Dim sArchiveFile$: sArchiveFile = sArchiveFolder & ".zip"
  If ZipFolder(sArchiveFolder, sArchiveFile) Then _
    CreateVersionArchive = sArchiveFile
    
SAFE_EXIT:
  Dim fso As New Scripting.FileSystemObject
  If fso.FolderExists(sArchiveFolder) Then _
    Call fso.DeleteFolder(sArchiveFolder)
End Function

Public Function ArchiveNameFor(prod As InfoProduct, DDate As Double) As String
  ArchiveNameFor = _
    prod.ArtifactHome & "\" & _
    SUBFOLDER_VERSION_ARCHIVE & "\" & _
    Format(DDate, "yyyymmdd") & " " & _
    prod.Name
End Function

Public Function ScanProducts(sTarget$) As Long
  Dim iCallback As New CB_AddProduct: Call iCallback.Init(AccessProducts)
  Call ForEachFolderRecursive(sTarget, iCallback, "ScanFolder")
  ScanProducts = iCallback.newCount_
End Function

' =====
Private Function GetSharedPrefix(target As VBIDE.VBComponent) As String
  Dim iShared As IteratorSharedModule: Set iShared = AccessShared.FindName(target.Name)
  If Not iShared Is Nothing Then _
    GetSharedPrefix = iShared.Path
End Function

Private Function GetAllSharedModules(sShared$) As Scripting.Dictionary
  Dim iShared As New CB_SharedModules: Call iShared.Init
  Call ForEachFileRecursive(sShared, iShared, "ProcessFile")
  Set GetAllSharedModules = iShared.data_
End Function

Private Function UpdateSharedUsage(ByRef iOut As IteratorSharedModule, dbComponents As DB_Components) As Collection
  Dim fso As New Scripting.FileSystemObject
  Dim iComp As IteratorComponent: Set iComp = dbComponents.IBegin
  Dim iData As New Scripting.Dictionary
  Do While Not iComp.IsDone
    Dim iManifest As InfoManifest: Set iManifest = iComp.ScanManifest
    If iManifest Is Nothing Then _
      GoTo NEXT_COMPONENT
    
    Dim sComponent$: sComponent = iComp.ItemName
    Dim vModule As Variant
    For Each vModule In iManifest.sharedContents_
      Dim sModuleName$: sModuleName = fso.GetBaseName(CStr(vModule))
      If Not iData.Exists(sModuleName) Then _
        Call iData.Add(sModuleName, New Collection)
      Call iData(sModuleName).Add(sComponent)
    Next vModule
NEXT_COMPONENT:
    Call iComp.Increment
  Loop
  
  Call iOut.GoFirst
  Do While Not iOut.IsDone
    Dim sName$: sName = iOut.Name
    If Not iData.Exists(sName) Then _
      GoTo NEXT_ROW
      
    Dim sItem As Variant
    Dim sComps$: sComps = vbNullString
    For Each sItem In iData(sName)
      If sComps <> vbNullString Then _
        sComps = sComps & ", "
      sComps = sComps & CStr(sItem)
    Next sItem
    iOut.UsedIn = sComps
    Call iData.Remove(sName)
NEXT_ROW:
    Call iOut.Increment
  Loop

  Dim oErrors As New Collection
  Dim aKey As Variant
  For Each aKey In iData
    Call oErrors.Add(CStr(aKey))
  Next aKey
  Set UpdateSharedUsage = oErrors
End Function
