VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "IteratorComponent"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

' _C_ - project components
Private Enum ComponentStruct
  [_First] = 1
  
  S_C_PRODUCT = 1
  S_C_ITEM_NAME = 2
  S_C_PATH_ARTIFACT = 3
  S_C_LOCAL_MANIFEST = 4
  S_C_BUILD_STATUS = 5
  S_C_BUILD_DATE = 6
  S_C_TESTS_TOTAL = 7
  S_C_TESTS_FAIL = 8
  S_C_TESTS_OK = 9
  
  [_Last] = 9
End Enum

Public row_ As Long
Private data_ As Excel.Worksheet

Public Sub Init(target As Excel.Worksheet, Optional tRow& = FIRST_ROW)
  Set data_ = target
  row_ = tRow
End Sub

Public Function Increment(Optional inc& = 1)
  If row_ + inc > 0 Then _
    row_ = row_ + inc
End Function

Public Function GoFirst()
  row_ = FIRST_ROW
End Function

Public Function GoLast()
  row_ = data_.Columns(S_C_PRODUCT).Find(vbNullString, LookAt:=xlWhole).Row - 1
End Function

Public Function IsDone() As Boolean
  IsDone = data_.Cells(row_, S_C_PRODUCT) = vbNullString
End Function

Public Function RemoveRow()
  Call data_.Rows(row_).Delete
End Function

Public Function Model() As InfoComponent
  Dim iProduct As IteratorProduct: Set iProduct = AccessProduct(Product)
  If iProduct Is Nothing Then _
    Exit Function
    
  Dim oRes As New InfoComponent
  If Not oRes.Init(ManifestFile, iProduct.SourcePath, iProduct.ArtifactPath, iProduct.Version) Then _
    Exit Function
  
  Call SyncManifest(Product, oRes.manifest_)
  Set Model = oRes
End Function

Public Function ScanManifest() As InfoManifest
  Dim iManifest As New InfoManifest
  If iManifest.Init(ManifestFile) Then _
    Set ScanManifest = iManifest
End Function

Public Function SyncManifest(sProduct$, iSource As InfoManifest)
  Dim fso As New Scripting.FileSystemObject
  Product = sProduct
  ItemName = iSource.properties_(MANIFEST_PROPS_NAME)
  Artifact = iSource.properties_(MANIFEST_PROPS_ARTIFACT)
  Manifest = fso.GetFileName(iSource.sPath_)
  Call UpdateItemLink
End Function

Public Function SyncBuildInfo(iSource As InfoBuild)
  BuildStatus = iSource.status_
  BuildDate = iSource.date_
  TestsTotal = iSource.tests_.count_
  TestsOK = iSource.tests_.success_
  TestsFail = iSource.tests_.failed_
End Function

Public Function SyncTestInfo(iSource As CDS_InfoTests)
  TestsTotal = iSource.count_
  TestsOK = iSource.success_
  TestsFail = iSource.failed_
End Function

Public Function ResetBuildStatus()
  If BuildStatus <> T_BS_FAILED Then _
    BuildStatus = T_BS_PENDING
End Function

Public Function SetBuildStatus(newVal As TBuildStatus)
  BuildStatus = newVal
  BuildDate = Now
End Function

Public Function UpdateItemLink()
  Call XLUpdateHyperlink(data_.Cells(row_, S_C_ITEM_NAME), ArtifactFile)
End Function

'===== Propertiy Get =====
Public Property Get ArtifactFile() As String
  Dim iProduct As IteratorProduct: Set iProduct = AccessProduct(Product)
  If iProduct Is Nothing Then _
    Exit Property
  ArtifactFile = CPath(data_.Cells(row_, S_C_PATH_ARTIFACT)).ToGlobal(iProduct.ArtifactPath).Text
End Property

Public Property Get ManifestFile() As String
  ManifestFile = CPath(Manifest).ToGlobal(SourcePath & "\" & SUBFOLDER_SCRIPT).Text
End Property

Public Property Get SkeletonFile() As String
  SkeletonFile = SourcePath & "\" & SUBFOLDER_SKELETON & "\" & ItemName
End Property

Public Property Get Product() As String
  Product = data_.Cells(row_, S_C_PRODUCT)
End Property

Public Property Get ItemName() As String
  ItemName = data_.Cells(row_, S_C_ITEM_NAME)
End Property

Public Property Get Artifact() As String
  Artifact = data_.Cells(row_, S_C_PATH_ARTIFACT)
End Property

Public Property Get Manifest() As String
  Manifest = data_.Cells(row_, S_C_LOCAL_MANIFEST)
End Property

Public Property Get BuildStatus() As TBuildStatus
  BuildStatus = data_.Cells(row_, S_C_BUILD_STATUS)
End Property

Public Property Get BuildDate() As Double
  BuildDate = data_.Cells(row_, S_C_BUILD_DATE)
End Property

Public Property Get TestsTotal() As Long
  TestsTotal = data_.Cells(row_, S_C_TESTS_TOTAL)
End Property

Public Property Get TestsFail() As Long
  TestsFail = data_.Cells(row_, S_C_TESTS_FAIL)
End Property

Public Property Get TestsOK() As Long
  TestsOK = data_.Cells(row_, S_C_TESTS_OK)
End Property

' ==== Property Let ====
Public Property Let Product(newVal$)
  data_.Cells(row_, S_C_PRODUCT) = newVal
End Property

Public Property Let ItemName(newVal$)
  data_.Cells(row_, S_C_ITEM_NAME) = newVal
End Property

Public Property Let Artifact(newVal$)
  data_.Cells(row_, S_C_PATH_ARTIFACT) = newVal
End Property

Public Property Let Manifest(newVal$)
  data_.Cells(row_, S_C_LOCAL_MANIFEST) = newVal
End Property

Public Property Let BuildStatus(newVal As TBuildStatus)
  data_.Cells(row_, S_C_BUILD_STATUS) = newVal
End Property

Public Property Let BuildDate(newVal As Double)
  data_.Cells(row_, S_C_BUILD_DATE) = newVal
End Property

Public Property Let TestsTotal(newVal&)
  data_.Cells(row_, S_C_TESTS_TOTAL) = newVal
End Property

Public Property Let TestsFail(newVal&)
  data_.Cells(row_, S_C_TESTS_FAIL) = newVal
End Property

Public Property Let TestsOK(newVal&)
  data_.Cells(row_, S_C_TESTS_OK) = newVal
End Property

' =======
Private Property Get SourcePath() As String
  Dim iProduct As IteratorProduct: Set iProduct = AccessProduct(Product)
  If iProduct Is Nothing Then _
    Exit Property
  SourcePath = Globals().SourceHome & "\" & iProduct.SourceHome
End Property
