Attribute VB_Name = "z_UIMessages"
' Messaging module
Option Private Module
Option Explicit

Public Enum MsgCode
  MSG_OK = 0
  
  EM_INVALID_SELECTION
  EM_SELF_MODIFICATION
  EM_IMPORT_FAILED
  EM_EXPORT_FAILED
  EM_MANIFEST_INVALID_FORMAT
  EM_INVALID_COMPONENT
  EM_SKELETON_FAILED
  EM_MISSING_MAKEFILE
  EM_MISSING_PRODUCT
  EM_MISSING_MANIFEST
  EM_INVALID_PRODUCT
  EM_MISSING_FILE
  EM_TESTS_FAILED
  EM_MAKEFILE_MISSING
  EM_MAKEFILE_LOADING_FAILED
  EM_PRODUCT_ALREADY_EXISTS
  
  IM_MAKEFILE_OK
  IM_MAKEFILE_FAILED
  IM_MANIFEST_OK
  IM_MANIFEST_FAILED
  IM_COMPILATION_OK
  IM_COMPILATION_FAILED
  IM_BUILD_OK
  IM_BUILD_FAILED
  IM_INSTALL_OK
  IM_INSTALL_FAILED
  IM_RELOAD_OK
  IM_RELOAD_FAILED
  IM_TESTS_NOT_FOUND
  IM_TESTS_COMPLETE
  IM_UPDATE_OK
  IM_UPDATE_FAILED
  IM_SHARED_INVALID
  IM_PRODUCT_ADDED
  IM_BATCH_BUILD
  IM_BATCH_INSTALL
  IM_SCAN_OK
  IM_UPDATE_SHARED_OK
  IM_UPDATE_SHARED_FAILED
  
  QM_CODE_DELETE_CONFIRM
End Enum

Private g_UI As API_UserInteraction

Public Function UserInteraction() As API_UserInteraction
  If g_UI Is Nothing Then _
    Set g_UI = New API_UserInteraction
  Set UserInteraction = g_UI
End Function

Public Function SetUserInteraction(newUI As API_UserInteraction)
  Set g_UI = newUI
End Function

Public Function UIShowMessage(theCode As MsgCode, ParamArray params() As Variant)
  Dim unwrapped As Variant: unwrapped = params
  unwrapped = FixForwardedParams(unwrapped)
  
  Select Case theCode
    Case EM_INVALID_SELECTION:        Call MsgBox("Invalid selection, please select single data item", vbExclamation)
    Case EM_SELF_MODIFICATION:        Call MsgBox("Cannot self-modify" & vbNewLine & "Please use renamed copy of this Workbook", vbExclamation)
    Case EM_IMPORT_FAILED:            Call MsgBox("Import failed", vbExclamation)
    Case EM_EXPORT_FAILED:            Call MsgBox("Export failed", vbExclamation)
    Case EM_MANIFEST_INVALID_FORMAT:  Call MsgBox("Invalid manifest format", vbExclamation)
    Case EM_INVALID_COMPONENT:        Call MsgBox("Cannot access selected component", vbExclamation)
    Case EM_INVALID_PRODUCT:          Call MsgBox("Cannot access selected product", vbExclamation)
    Case EM_SKELETON_FAILED:          Call MsgBox("Unable to produce skeleton", vbExclamation)
    Case EM_MISSING_MAKEFILE:         Call MsgBox(Fmt("Missing makefile: {1}", unwrapped), vbExclamation)
    Case EM_MISSING_PRODUCT:          Call MsgBox(Fmt("Missing product data entry: {1}", unwrapped), vbExclamation)
    Case EM_MISSING_MANIFEST:         Call MsgBox(Fmt("Missing manifest file: {1}", unwrapped), vbExclamation)
    Case EM_MISSING_FILE:             Call MsgBox(Fmt("Missing target file: {1}", unwrapped), vbExclamation)
    Case EM_TESTS_FAILED:             Call MsgBox("Failed to run the tests. See log output for more info", vbExclamation)
    Case EM_MAKEFILE_MISSING:         Call MsgBox(Fmt("Missing makefile: {1}", unwrapped), vbExclamation)
    Case EM_MAKEFILE_LOADING_FAILED:  Call MsgBox("Cannot load makefile contents", vbExclamation)
    Case EM_PRODUCT_ALREADY_EXISTS:   Call MsgBox(Fmt("Product with this name already exists: ", unwrapped), vbExclamation)
      
    Case IM_MAKEFILE_OK:              Call MsgBox("Makefile is valid", vbInformation)
    Case IM_MAKEFILE_FAILED:          Call MsgBox(Fmt("Makefile is not valid!" & vbNewLine & vbNewLine & "{1}", unwrapped), vbExclamation)
    Case IM_MANIFEST_OK:              Call MsgBox("Manifest is valid", vbInformation)
    Case IM_MANIFEST_FAILED:          Call MsgBox(Fmt("Manifest is not valid!" & vbNewLine & vbNewLine & "{1}", unwrapped), vbExclamation)
    Case IM_COMPILATION_OK:           Call MsgBox("All components are compilable", vbInformation)
    Case IM_COMPILATION_FAILED:       Call MsgBox(Fmt("Some components are broken!" & vbNewLine & vbNewLine & "{1}", unwrapped), vbExclamation)
    Case IM_RELOAD_OK:                Call MsgBox("All components updated", vbInformation)
    Case IM_RELOAD_FAILED:            Call MsgBox(Fmt("Some components could not be updated!" & vbNewLine & vbNewLine & "{1}", unwrapped), vbExclamation)
    Case IM_BUILD_OK:                 Call MsgBox(Fmt("Build successfull" & vbNewLine & "Time elapsed: {1}", unwrapped), vbInformation)
    Case IM_BUILD_FAILED:             Call MsgBox("Build failed. See debug output", vbExclamation)
    Case IM_INSTALL_OK:               Call MsgBox(Fmt("Installationg successfull" & vbNewLine & "Time elapsed: {1}", unwrapped), vbInformation)
    Case IM_INSTALL_FAILED:           Call MsgBox("Installation failed. See debug output", vbExclamation)
    Case IM_TESTS_NOT_FOUND:          Call MsgBox("No tests available for this component", vbInformation)
    Case IM_UPDATE_OK:                Call MsgBox("Target update success", vbInformation)
    Case IM_UPDATE_FAILED:            Call MsgBox("Target update failed", vbExclamation)
    Case IM_UPDATE_SHARED_OK:         Call MsgBox(Fmt("Target shared modules updated: {1}" & vbNewLine & "{2}", unwrapped), vbInformation)
    Case IM_UPDATE_SHARED_FAILED:     Call MsgBox("Failed to compile after update", vbExclamation)
    Case IM_SHARED_INVALID:           Call MsgBox(Fmt("Unknown shared modules" & vbNewLine & "{1}", unwrapped), vbExclamation)
    Case IM_PRODUCT_ADDED:            Call MsgBox(Fmt("Product added: {1}", unwrapped), vbInformation)
    Case IM_BATCH_INSTALL:            Call MsgBox(Fmt("Install all products complete: {1} / {2}" & vbNewLine & "Time elapsed: {3}", unwrapped), vbInformation)
    Case IM_BATCH_BUILD:              Call MsgBox(Fmt("Build all products complete: {1} / {2}" & vbNewLine & "Time elapsed: {3}", unwrapped), vbInformation)
    Case IM_SCAN_OK:                  Call MsgBox(Fmt("Scanning makefiles complete. Found {1} new products", unwrapped), vbInformation)
    Case IM_TESTS_COMPLETE
      Call MsgBox(Fmt("Testing finished: {1} | {2} / {3}" & vbNewLine & "Time elapsed: {4}", unwrapped), IIf(unwrapped(2) = 0, vbInformation, vbExclamation))
      
    Case Else:    Call MsgBox("Invalid message code", vbCritical)
  End Select
End Function

Public Function UIAskQuestion(theCode As MsgCode, ParamArray params() As Variant) As Boolean
  Dim unwrapped As Variant: unwrapped = params
  unwrapped = FixForwardedParams(unwrapped)
  
  Dim answer&: answer = vbNo
  Select Case theCode
  Case QM_CODE_DELETE_CONFIRM
    answer = MsgBox("Are you sure you want to delete ALL macros from target file?", vbYesNo + vbQuestion)
    
  Case Else
    Call MsgBox("Invalid message code", vbCritical)
  End Select
  UIAskQuestion = answer = vbYes
End Function
