VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "s_Config"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

Private fso_ As Scripting.FileSystemObject
Private config_ As API_Config

Public Function Setup()
  ' Mandatory setup function
  Set fso_ = New Scripting.FileSystemObject
  Set config_ = New API_Config
  Call EnsureFolderExists(Dev_GetTestFolder)
End Function

Public Function Teardown()
  ' Mandatory teardown function
  Call fso_.DeleteFolder(Dev_GetTestFolder)
End Function

Public Function t_BasicDictionary()
  On Error GoTo PROPAGATE_ERROR

  Call Dev_NewCase("Empty config")
  Call Dev_ExpectFalse(config_.Contains(""))
  Call Dev_ExpectFalse(config_.Contains("1"))
  Call Dev_ExpectEQ(42, config_.GetValue("1", 42))
  Call Dev_ExpectEQ("", config_.GetValue("1"))
  
  Call Dev_NewCase("Basic type")
  Call config_.SetValue("1", 42)
  Call Dev_ExpectTrue(config_.Contains("1"))
  Call Dev_ExpectEQ(42, config_.GetValue("1"))
  Call config_.Clear
  Call Dev_ExpectFalse(config_.Contains("1"))
  
  Call Dev_NewCase("Double set")
  Call config_.SetValue("1", 1)
  Call config_.SetValue("1", 2)
  Call Dev_ExpectEQ(2, config_.GetValue("1"))
  Call config_.Clear
  
  Call Dev_NewCase("Object type")
  Call config_.SetValue("1", CColl(1, 2))
  Call Dev_ExpectEQ(CColl(1, 2), config_.GetValue("1"))
  Call Dev_ExpectEQ(CColl(1, 2, 3), config_.GetValue("2", CColl(1, 2, 3)))
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_FileIO()
  On Error GoTo PROPAGATE_ERROR
  
  Call Dev_NewCase("Save to file")
  Call config_.SetValue("0", "")
  Call config_.SetValue("1", 1)
  Call config_.SetValue("2", "42")
  Call config_.SetValue("3", CColl(1, 2))
  Dim sTestFile$: sTestFile = Dev_GetTestFolder & "\test.json"
  Call Dev_AssertTrue(config_.SaveToFile(sTestFile))
  
  Call Dev_NewCase("Load from file")
  Dim iLoad As New API_Config
  Call Dev_AssertTrue(iLoad.LoadFromFile(sTestFile))
  Call Dev_ExpectEQ(config_.GetValue("0"), iLoad.GetValue("0"))
  Call Dev_ExpectEQ(config_.GetValue("1"), iLoad.GetValue("1"))
  Call Dev_ExpectEQ(config_.GetValue("2"), iLoad.GetValue("2"))
  Call Dev_ExpectEQ(config_.GetValue("3"), iLoad.GetValue("3"))
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_JSON_IO()
  On Error GoTo PROPAGATE_ERROR
  
  Call Dev_NewCase("Save and Load JSON")
  Call config_.SetValue("0", "")
  Call config_.SetValue("1", 1)
  Call config_.SetValue("2", "42")
  Call config_.SetValue("3", CColl(1, 2))
  Call config_.SetValue("4", True)
  Dim sJson$: sJson = config_.SaveToJSON()

  Dim iLoad As New API_Config
  Call Dev_AssertTrue(iLoad.LoadFromJSON(sJson))
  Call Dev_ExpectEQ(config_.GetValue("0"), iLoad.GetValue("0"))
  Call Dev_ExpectEQ(config_.GetValue("1"), iLoad.GetValue("1"))
  Call Dev_ExpectEQ(config_.GetValue("2"), iLoad.GetValue("2"))
  Call Dev_ExpectEQ(config_.GetValue("3"), iLoad.GetValue("3"))
  Call Dev_ExpectEQ(config_.GetValue("4"), iLoad.GetValue("4"))
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function
