VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "s_ExVBA"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
' ===== Test suite for date Parsing =======
Option Explicit

Public Function Setup()
  ' Mandatory setup function
End Function

Public Function Teardown()
  ' Mandatory teardown function
End Function

Public Function t_CreateCollection()
  On Error GoTo PROPAGATE_ERROR
  
  Call Dev_ExpectEQ(New Collection, CColl(), "Empty collection")
  
  Call Dev_NewCase("Valid collection")
  Dim iExpect As New Collection
  Call iExpect.Add(1): Call iExpect.Add("13"): Call iExpect.Add(3): Call iExpect.Add(1)
  Call Dev_ExpectEQ(iExpect, CColl(1, "13", 3, 1))
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_CSet()
  On Error GoTo PROPAGATE_ERROR
  
  Call Dev_ExpectEQ(New Scripting.Dictionary, CSet(), "Empty set")
  
  Call Dev_NewCase("Valid set")
  Dim iExpect As New Scripting.Dictionary
  Call iExpect.Add(1, 0): Call iExpect.Add("13", 0): Call iExpect.Add(3, 0)
  Call Dev_ExpectEQ(iExpect, CSet(1, "13", 3))
  
  Call Dev_NewCase("Duplicate elements")
  Call Dev_ExpectEQ(iExpect, CSet(1, "13", 3, 1, 3))
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_CreateDictionary()
  On Error GoTo PROPAGATE_ERROR
  
  Call Dev_ExpectEQ(New Scripting.Dictionary, CDict(), "Empty dict")
  
  Call Dev_NewCase("Valid set")
  Dim iExpect As New Scripting.Dictionary
  Call iExpect.Add(1, 42): Call iExpect.Add("13", 1337): Call iExpect.Add(3, CDict())
  Call Dev_ExpectEQ(iExpect, CDict(1, 42, "13", 1337, 3, CDict()))
  
  Call Dev_NewCase("Invalid number of args")
  On Error Resume Next
  Set iExpect = CDict(1, "13", 3)
  Call Dev_ExpectAnyError
  On Error GoTo PROPAGATE_ERROR
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_UniqueList()
  On Error GoTo PROPAGATE_ERROR
  
  Dim uList As Scripting.Dictionary
  
  Call Dev_ExpectEQ(0, UniqueList.Count, "Empty list")
  
  Call Dev_NewCase("Simple list")
  Set uList = UniqueList(13, 37, 3)
  Call Dev_AssertEQ(3, uList.Count)
  Call Dev_ExpectEQ(1, uList(13))
  Call Dev_ExpectEQ(2, uList(37))
  Call Dev_ExpectEQ(3, uList(3))
  
  Call Dev_NewCase("Repeating elements")
  Set uList = UniqueList(13, 13, 3)
  Call Dev_AssertEQ(2, uList.Count)
  Call Dev_ExpectEQ(1, uList(13))
  Call Dev_ExpectEQ(3, uList(3))
  
  Call Dev_NewCase("Mixed list")
  Set uList = UniqueList(13, "37", 3.1)
  Call Dev_AssertEQ(3, uList.Count)
  Call Dev_ExpectEQ(1, uList(13))
  Call Dev_ExpectEQ(2, uList("37"))
  Call Dev_ExpectEQ(3, uList(3.1))
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_ArraySize()
  On Error GoTo PROPAGATE_ERROR
  
  Call Dev_ExpectEQ(VBA_INVALID_SIZE, ArraySize(Nothing, 1), "Nothing")
  Call Dev_ExpectEQ(VBA_INVALID_SIZE, ArraySize(1, 1), "Non-array")
  Call Dev_ExpectEQ(VBA_INVALID_SIZE, ArraySize(New Collection, 1), "Object")
  
  Dim unallocArr() As Variant
  Call Dev_ExpectEQ(VBA_INVALID_SIZE, ArraySize(unallocArr, 1), "Unallocated")
  
  Dim arr1() As Variant
  ReDim arr1(0 To 4)
  Call Dev_ExpectEQ(5, ArraySize(arr1, 1), "1D Array 0-based")
  Call Dev_ExpectEQ(VBA_INVALID_SIZE, ArraySize(arr1, 2), "Out of bounds dimension")
  Call Dev_ExpectEQ(VBA_INVALID_SIZE, ArraySize(arr1, -1), "Invalid dimension")
  
  ReDim arr1(1 To 5)
  Call Dev_ExpectEQ(5, ArraySize(arr1, 1), "1D Array 1-based")
  
  Call Dev_NewCase("2D array")
  Dim arr2() As Variant
  ReDim arr2(1 To 5, 1 To 7)
  Call Dev_ExpectEQ(5, ArraySize(arr2, 1))
  Call Dev_ExpectEQ(7, ArraySize(arr2, 2))
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_ArrayToIndex()
  On Error GoTo PROPAGATE_ERROR
  
  Call Dev_ExpectNothing(ArrayToIndex(Array()), "Not allocated")
  
  Call Dev_NewCase("Not initialized")
  Dim arr(1 To 2) As Variant
  Dim indicies As New Scripting.Dictionary
  indicies(Empty) = 1
  Call Dev_ExpectEQ(indicies, ArrayToIndex(arr))
  
  Call Dev_NewCase("Initialized unique elements")
  Call indicies.RemoveAll
  arr(1) = 1337: indicies(1337) = 1
  arr(2) = "leet": indicies("leet") = 2
  Call Dev_ExpectEQ(indicies, ArrayToIndex(arr))
  
  Call Dev_NewCase("Initialized duplicate elements")
  Call indicies.RemoveAll
  arr(1) = 1337: indicies(1337) = 1
  arr(2) = 1337
  Call Dev_ExpectEQ(indicies, ArrayToIndex(arr))
  
  Call Dev_NewCase("Custom base")
  Dim arr2(2 To 5) As Variant
  Call indicies.RemoveAll
  indicies(Empty) = 2
  Call Dev_ExpectEQ(indicies, ArrayToIndex(arr2))
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_IsArrayAllocated()
  On Error GoTo PROPAGATE_ERROR
  
  Call Dev_ExpectFalse(IsArrayAllocated(Nothing), "Nothing")
  Call Dev_ExpectFalse(IsArrayAllocated(1), "Non-array")
  Call Dev_ExpectFalse(IsArrayAllocated(New Collection), "Object")
  
  Call Dev_NewCase("Unallocated")
  Dim unallocArr() As Variant
  Call Dev_ExpectFalse(IsArrayAllocated(unallocArr))
  Call Dev_ExpectNoError
  
  Dim arr1(1 To 5) As Variant
  Call Dev_ExpectTrue(IsArrayAllocated(arr1), "Allocated empty array")
  
  Dim arr2 As Variant: arr2 = Array(1, 2, 3)
  Call Dev_ExpectTrue(IsArrayAllocated(arr2), "Allocated full array")
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_FixForwardedParams()
  On Error GoTo PROPAGATE_ERROR
  
  Call Dev_ExpectFalse(IsArrayAllocated(TestForwardParams()), "No params")
  Call Dev_ExpectEQ(Array(1), TestForwardParams(1), "Simple arg")
  Call Dev_ExpectEQ(Array(1, "123"), TestForwardParams(1, "123"), "Multiple arg")
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_CompareApproximate()
  On Error GoTo PROPAGATE_ERROR
  
  On Error Resume Next
  Call Dev_ExpectEQ(0, CompareApproximate("a", "b", 0))
  Call Dev_ExpectError(SYS_ERR_TYPE_MISMATCH, "Invalid type comparison")
  On Error GoTo PROPAGATE_ERROR
  
  Call Dev_NewCase("Precision equals")
  Call Dev_ExpectEQ(0, CompareApproximate(0, 0, 0))
  Call Dev_ExpectEQ(0, CompareApproximate(1, 1, 0))
  Call Dev_ExpectEQ(0, CompareApproximate(1.1, 1.1, 0))
  Call Dev_ExpectNE(0, CompareApproximate(1.1, 1, 1))
  Call Dev_ExpectEQ(0, CompareApproximate(1.1, 1, 0))
  Call Dev_ExpectEQ(0, CompareApproximate(11, 10, -1))
  Call Dev_ExpectEQ(0, CompareApproximate(111, 100, -2))
  Call Dev_ExpectEQ(11, CompareApproximate(111, 100, -1))
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_CompareDeepBasic()
  On Error GoTo PROPAGATE_ERROR
  
  Call Dev_NewCase("Basic type equals")
  Call Dev_ExpectEQ(0, CompareDeep(True, True))
  Call Dev_ExpectEQ(0, CompareDeep(0, 0))
  Call Dev_ExpectEQ(0, CompareDeep(0&, 0&))
  Call Dev_ExpectEQ(0, CompareDeep(1.1, 1.1))
  Call Dev_ExpectEQ(0, CompareDeep(1.1!, 1.1!))
  Call Dev_ExpectEQ(0, CompareDeep("", ""))
  
  Call Dev_NewCase("Basic type compare")
  Call Dev_ExpectNE(0, CompareDeep(False, True))
  Call Dev_ExpectNE(0, CompareDeep(True, False))
  Call Dev_ExpectEQ(1, CompareDeep(2, 1))
  Call Dev_ExpectEQ(2, CompareDeep(3, 1))
  Call Dev_ExpectEQ(-2, CompareDeep(1, 3))
  Call Dev_ExpectEQ(2, CompareDeep(3&, 1&))
  Call Dev_ExpectEQ(-2, CompareDeep(1&, 3&))
  Call Dev_ExpectAEQ(2, CompareDeep(3.1, 1.1), 0)
  Call Dev_ExpectAEQ(-2, CompareDeep(1.1, 3.1), 0)
  Call Dev_ExpectAEQ(1.9, CompareDeep(3, 1.1), 1)
  Call Dev_ExpectAEQ(2, CompareDeep(3.1!, 1.1!), 0)
  Call Dev_ExpectAEQ(-2, CompareDeep(1.1!, 3.1!), 0)
  Call Dev_ExpectEQ(2, CompareDeep("3", "1"))
  Call Dev_ExpectEQ(-2, CompareDeep("1", "3"))
  
  Call Dev_NewCase("Basic type cross compare")
  Call Dev_ExpectEQ(0, CompareDeep(0, False))
  Call Dev_ExpectEQ(0, CompareDeep(False, 0))
  Call Dev_ExpectEQ(0, CompareDeep(-1, True))
  Call Dev_ExpectEQ(0, CompareDeep(True, -1))
  Call Dev_ExpectEQ(1, CompareDeep(0, ""))
  Call Dev_ExpectEQ(1, CompareDeep("", 0))
  Call Dev_ExpectEQ(0, CompareDeep(0, "0"))
  Call Dev_ExpectEQ(0, CompareDeep("0", 0))
  Call Dev_ExpectEQ(0, CompareDeep(1, 1&))
  Call Dev_ExpectEQ(0, CompareDeep(1&, 1))
  Call Dev_ExpectEQ(0, CompareDeep(1, 1#))
  Call Dev_ExpectEQ(0, CompareDeep(1#, 1))
  Call Dev_ExpectAEQ(0, CompareDeep(1, 1!), 1)
  Call Dev_ExpectAEQ(0, CompareDeep(1!, 1), 1)
  Call Dev_ExpectAEQ(0, CompareDeep(1.1, 1.1!), 1)
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_CompareDeepObject()
  On Error GoTo PROPAGATE_ERROR
  
  Call Dev_ExpectEQ(0, CompareDeep(Nothing, Nothing), "Nothing compare")
  
  On Error Resume Next
  Call CompareDeep(ThisWorkbook, ThisWorkbook.Application)
  Call Dev_ExpectError(SYS_ERR_INVALID_OPERATION, "Mismatched type comparison")
  On Error GoTo PROPAGATE_ERROR
  
  Call Dev_ExpectEQ(0, CompareDeep(Me, Me), "Compare equal object pointers")
  Call Dev_ExpectEQ(1, CompareDeep(Me, New s_ExVBA), "Compare different object pointers")
  
  Call Dev_NewCase("Valid object compare")
  Dim obj1 As New TestCustomObject: obj1.data_ = 2
  Dim obj2 As New TestCustomObject: obj2.data_ = 1
  Call Dev_ExpectEQ(1, CompareDeep(obj1, obj2))
  obj2.data_ = 2
  Call Dev_ExpectEQ(0, CompareDeep(obj1, obj2))
  obj2.data_ = 3
  Call Dev_ExpectEQ(-1, CompareDeep(obj1, obj2))
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_CompareDeepCollection()
  On Error GoTo PROPAGATE_ERROR
  
  Dim colEmpty As New Collection
  Dim colEmpty2 As New Collection
  Dim col1 As New Collection: Call col1.Add(1): Call col1.Add(2)
  Dim col2 As New Collection: Call col2.Add(2): Call col2.Add(1)
  Dim col3 As New Collection: Call col3.Add(1): Call col3.Add(2)
  Dim col4 As New Collection: Call col4.Add(1): Call col4.Add(2): Call col4.Add(3)
  
  Call Dev_ExpectEQ(0, CompareDeep(colEmpty, colEmpty2), "Empty comparison")
  Call Dev_ExpectEQ(0, CompareDeep(colEmpty, colEmpty), "Empty self comparison")
  
  Call Dev_NewCase("Simple collection comparison")
  Call Dev_ExpectEQ(0, CompareDeep(col1, col1))
  Call Dev_ExpectEQ(0, CompareDeep(col1, col3))
  Call Dev_ExpectNE(0, CompareDeep(col1, col2))
  Call Dev_ExpectNE(0, CompareDeep(col1, col4))
  
  Call col1.Add("1")
  Call col3.Add(1)
  Call Dev_ExpectEQ(0, CompareDeep(col1, col3), "Mixed comparison")
  
  Dim obj1 As New TestCustomObject: obj1.data_ = 1
  Dim obj2 As New TestCustomObject: obj2.data_ = 1
  Call col1.Add(obj1)
  Call col3.Add(obj2)
  Call Dev_ExpectEQ(0, CompareDeep(col1, col3), "Object collection comparison")
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_CompareDeepDictionary()
  On Error GoTo PROPAGATE_ERROR
  
  Dim dictEmpty As New Scripting.Dictionary
  Dim dictEmpty2 As New Scripting.Dictionary
  Dim d1 As New Scripting.Dictionary: Call d1.Add(1, 1): Call d1.Add(2, 2)
  Dim d2 As New Scripting.Dictionary: Call d2.Add(1, 2): Call d2.Add(2, 1)
  Dim d3 As New Scripting.Dictionary: Call d3.Add(1, 1): Call d3.Add(2, 2)
  Dim d4 As New Scripting.Dictionary: Call d4.Add("1", 1): Call d4.Add("2", 2)
  Dim d5 As New Scripting.Dictionary: Call d5.Add(1, 1): Call d5.Add(2, 2): Call d5.Add(3, 3)
  
  Call Dev_ExpectEQ(0, CompareDeep(dictEmpty, dictEmpty2), "Empty comparison")
  Call Dev_ExpectEQ(0, CompareDeep(dictEmpty, dictEmpty), "Empty self comparison")
  
  Call Dev_NewCase("Dictionary compare")
  Call Dev_ExpectEQ(0, CompareDeep(d1, d1))
  Call Dev_ExpectEQ(0, CompareDeep(d1, d3))
  Call Dev_ExpectNE(0, CompareDeep(d1, d2))
  Call Dev_ExpectNE(0, CompareDeep(d1, d4))
  Call Dev_ExpectNE(0, CompareDeep(d1, d5))
  
  Call d1.Add(4, "1")
  Call d3.Add(4, 1)
  Call Dev_ExpectEQ(0, CompareDeep(d1, d3), "Mixed comparison")
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_CompareDeepArray()
  On Error GoTo PROPAGATE_ERROR
  
  Dim unallocArr() As Variant
  Dim arr1 As Variant: arr1 = Array(1, 2, 3)
  Dim arr2 As Variant: arr2 = Array(2, 1, 3)
  Dim arr3 As Variant: arr3 = Array(1, 2, 3)
  Dim arr4 As Variant: arr4 = Array("1", 2, 3)
  Dim arr5 As Variant: arr5 = Array("1", 2, 3, 4)
  
  Call Dev_ExpectEQ(0, CompareDeep(unallocArr, unallocArr), "Unallocated comparison")
  
  Call Dev_NewCase("Array compare")
  Call Dev_ExpectEQ(0, CompareDeep(arr1, arr1))
  Call Dev_ExpectEQ(0, CompareDeep(arr1, arr3))
  Call Dev_ExpectNE(0, CompareDeep(arr1, arr2))
  Call Dev_ExpectEQ(0, CompareDeep(arr1, arr4))
  Call Dev_ExpectNE(0, CompareDeep(arr1, arr5))
  
  Call Dev_NewCase("Compound array compare")
  Dim arr6 As Variant: arr6 = Array(Array(1), Array(2, 3), Array("3"))
  Call Dev_ExpectEQ(0, CompareDeep(arr6, arr6))
  Call Dev_ExpectNE(0, CompareDeep(arr1, arr6))
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_ExtractTimestamp()
  On Error GoTo PROPAGATE_ERROR
  
  Call Dev_ExpectEQ(0, ExtractTimestamp(""), "Empty string")
  Call Dev_ExpectEQ(0, ExtractTimestamp("INVALID"), "Invalid text")
  Call Dev_ExpectEQ(0, ExtractTimestamp("2020-13-99"), "Invalid timestamp")
  Call Dev_ExpectEQ(DateSerial(2020, 12, 30), ExtractTimestamp("2020-12-30"), "Short date")
  Call Dev_ExpectEQ(0, ExtractTimestamp("2020-12-30: TestMsg"), "Short date and Msg")
  
  Call ExtractTimestamp("2020-13-99")
  Call Dev_ExpectNoError("No error polution")
  
  Call Dev_NewCase("Full time date")
  Call Dev_ExpectEQ(CDbl(VBA.TimeValue("11:59:59") + VBA.DateValue("2020-12-30")), ExtractTimestamp("2020-12-30 11:59:59"))
  
  Call Dev_NewCase("Full time date and Msg")
  Call Dev_ExpectEQ(CDbl(VBA.TimeValue("11:59:59") + VBA.DateValue("2020-12-30")), ExtractTimestamp("2020-12-30 11:59:59: TestMsg"))
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_TrimTimestamp()
  On Error GoTo PROPAGATE_ERROR
  
  Call Dev_ExpectEQ(vbNullString, TrimTimestamp(""), "Empty string")
  Call Dev_ExpectEQ("INVALID", TrimTimestamp("INVALID"), "Invalid text")
  Call Dev_ExpectEQ("2020-13-99", TrimTimestamp("2020-13-99"), "Invalid timestamp")
  Call Dev_ExpectEQ(vbNullString, TrimTimestamp("2020-12-30"), "Short date")
  Call Dev_ExpectEQ("2020-12-30: TestMsg", TrimTimestamp("2020-12-30: TestMsg"), "Short date and Msg")
  
  Call Dev_NewCase("Full time date")
  Call Dev_ExpectEQ(vbNullString, TrimTimestamp("2020-12-30 11:59:59"))
  
  Call Dev_NewCase("Full time date and Msg")
  Call Dev_ExpectEQ("TestMsg", TrimTimestamp("2020-12-30 11:59:59: TestMsg"))
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_Fmt()
  On Error GoTo PROPAGATE_ERROR
  
  Call Dev_ExpectEQ(vbNullString, Fmt(vbNullString), "Empty string")
  Call Dev_ExpectEQ("{1}", Fmt("{1}"), "Missing arg")
  Call Dev_ExpectEQ("{0}", Fmt("{0}", 1337), "Invalid indexing")
  Call Dev_ExpectEQ("{2}", Fmt("{2}", 1337), "Missing index marker")
  Call Dev_ExpectEQ("1337 12 1337", Fmt("{1} {2} {1}", 1337, 12), "Miltiple markers")
  Call Dev_ExpectEQ("1337 12 1337 {3}", Fmt("{1} {2} {1} {3}", 1337, 12), "Partially valid")
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_PrintBasic()
  On Error GoTo PROPAGATE_ERROR
  
  Call Dev_ExpectEQ("", PrettyPrint(vbNullString), "Empty string")
  Call Dev_ExpectEQ("[Nothing]", PrettyPrint(Nothing), "Nothing")
  Call Dev_ExpectEQ("[Null]", PrettyPrint(Null), "Not initialized variant")
  Call Dev_ExpectEQ("abc", PrettyPrint("abc"), "String")
  Call Dev_ExpectEQ("0", PrettyPrint(0), "Ineger")
  Call Dev_ExpectEQ(CStr(3.14), PrettyPrint(3.14), "Float")
  Call Dev_ExpectEQ("True", PrettyPrint(True), "Boolean")
  
  Dim vNotInit As Variant
  Call Dev_ExpectEQ("[Empty]", PrettyPrint(vNotInit), "Not initialized variant")
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_PrintObject()
  On Error GoTo PROPAGATE_ERROR
  
  Call Dev_ExpectEQ("[Workbook]", PrettyPrint(ThisWorkbook), "Non-stringable type")
  Call Dev_ExpectEQ("Microsoft Excel", PrettyPrint(ThisWorkbook.Application), "Stringable type")
  
  Dim strObject As New TestCustomObject: strObject.data_ = 2
  Call Dev_ExpectEQ("2", PrettyPrint(strObject), "Custom object provides valid callback")
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_PrintCollection()
  On Error GoTo PROPAGATE_ERROR
  
  Dim testCol As New Collection
  Call Dev_ExpectEQ("$Collection []", PrettyPrint(testCol), "Empty collection")
  
  Call Dev_NewCase("Basic elements")
  Call testCol.Add(1): Call testCol.Add(2)
  Dim sResult$
  sResult = "$Collection [" & vbNewLine & _
              Indent(1) & "1," & vbNewLine & _
              Indent(1) & "2" & vbNewLine & _
            "]"
  Call Dev_ExpectEQ(sResult, PrettyPrint(testCol))
  
  Call Dev_NewCase("Compound elements")
  Dim intCol As New Collection: Call intCol.Add(3)
  Call testCol.Add(intCol)
  sResult = "$Collection [" & vbNewLine & _
              Indent(1) & "1," & vbNewLine & _
              Indent(1) & "2," & vbNewLine & _
              Indent(1) & "$Collection [" & vbNewLine & _
                Indent(2) & "3" & vbNewLine & _
              Indent(1) & "]" & vbNewLine & _
            "]"
  Call Dev_ExpectEQ(sResult, PrettyPrint(testCol))
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_PrintDictionary()
  On Error GoTo PROPAGATE_ERROR
  
  Dim testDict As New Scripting.Dictionary
  Call Dev_ExpectEQ("$Dictionary []", PrettyPrint(testDict), "Empty dictionary")
  
  Call Dev_NewCase("Basic elements")
  Call testDict.Add(1, "a"): Call testDict.Add(2, "b")
  Dim sResult$
  sResult = "$Dictionary [" & vbNewLine & _
              Indent(1) & "1: a," & vbNewLine & _
              Indent(1) & "2: b" & vbNewLine & _
            "]"
  Call Dev_ExpectEQ(sResult, PrettyPrint(testDict))
  
  Call Dev_NewCase("Compound elements")
  Dim intDict As New Scripting.Dictionary: Call intDict.Add(3, "c")
  Call testDict.Add(3, intDict)
  sResult = "$Dictionary [" & vbNewLine & _
              Indent(1) & "1: a," & vbNewLine & _
              Indent(1) & "2: b," & vbNewLine & _
              Indent(1) & "3: $Dictionary [" & vbNewLine & _
                Indent(2) & "3: c" & vbNewLine & _
              Indent(1) & "]" & vbNewLine & _
            "]"
  Call Dev_ExpectEQ(sResult, PrettyPrint(testDict))
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_PrintArray()
  On Error GoTo PROPAGATE_ERROR
  
  Dim testArr() As Variant
  Call Dev_ExpectEQ("$Array []", PrettyPrint(testArr), "Unallocated array")
  
  Call Dev_NewCase("Uninit array")
  ReDim testArr(1 To 2)
  Dim sResult$
  sResult = "$Array [" & vbNewLine & _
              Indent(1) & "[Empty]," & vbNewLine & _
              Indent(1) & "[Empty]" & vbNewLine & _
            "]"
  Call Dev_ExpectEQ(sResult, PrettyPrint(testArr))
  
  Call Dev_NewCase("Basic elements")
  testArr(1) = 1
  testArr(2) = 2
  sResult = "$Array [" & vbNewLine & _
              Indent(1) & "1," & vbNewLine & _
              Indent(1) & "2" & vbNewLine & _
            "]"
  Call Dev_ExpectEQ(sResult, PrettyPrint(testArr))
            
  Call Dev_NewCase("Compound elements")
  
  Set testArr(2) = New Collection
  sResult = "$Array [" & vbNewLine & _
              Indent(1) & "1," & vbNewLine & _
              Indent(1) & "$Collection []" & vbNewLine & _
            "]"
  Call Dev_ExpectEQ(sResult, PrettyPrint(testArr))
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_CommonPrefixLength()
  On Error GoTo PROPAGATE_ERROR
  
  Call Dev_NewCase("Empty")
  Call Dev_ExpectEQ(0, CommonPrefixLength("", ""))
  Call Dev_ExpectEQ(0, CommonPrefixLength("abc", ""))
  Call Dev_ExpectEQ(0, CommonPrefixLength("", "abc"))
  
  Call Dev_NewCase("Valid prefix")
  Call Dev_ExpectEQ(3, CommonPrefixLength("123", "123"))
  Call Dev_ExpectEQ(3, CommonPrefixLength("123", "1234"))
  Call Dev_ExpectEQ(3, CommonPrefixLength("1234", "123"))
  Call Dev_ExpectEQ(11, CommonPrefixLength("C:\Windows\Test", "C:\Windows\Process"))
  
  Call Dev_NewCase("Compare mode")
  Call Dev_ExpectEQ(0, CommonPrefixLength("ABC", "abc", vbBinaryCompare))
  Call Dev_ExpectEQ(3, CommonPrefixLength("ABC", "abc", vbTextCompare))
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_DictionaryAbsorbShallow()
  On Error GoTo PROPAGATE_ERROR
  
  Dim iSource As New Scripting.Dictionary
  Dim iDestination As New Scripting.Dictionary
  iSource.Item(1) = 1337
  iSource.Item(42) = 43
  iDestination.Item(1) = 42
  Set iSource.Item(2) = CColl(1, 2, 3)
  
  Call DictionaryAbsorbShallow(iDestination, iSource)
  Call Dev_AssertEQ(3, iDestination.Count, "Count elements")
  Call Dev_ExpectEQ(43, iDestination(42), "Transfer basic values")
  Call Dev_ExpectEQ(42, iDestination(1), "Do not overwrite common keys")
  Call Dev_ExpectEQ(CColl(1, 2, 3), iDestination(2), "Transfer complex elements")
  
  Call Dev_NewCase("Shallow copy")
  iSource.Item(42) = 44
  Call iSource.Item(2).Add(4)
  Call Dev_ExpectEQ(43, iDestination(42))
  Call Dev_ExpectEQ(CColl(1, 2, 3, 4), iDestination(2))
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

' =====
Private Function TestForwardParams(ParamArray params() As Variant) As Variant
  Dim val As Variant: val = params
  TestForwardParams = FixForwardedParams(val)
End Function
