VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "s_ExWinAPI"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

' TODO:
' Public Function ForEachFileRecursive(sTargetFolder$, oCallback As Object, sFuncName$)
' Public Function UnzipFile(sTargetFile$, sDestinationFolder$) As Boolean
' Public Function ZipFolder(sTargetFolder$, sDestinationFile$) As Boolean
' Public Function ListFilesIn(sTargetFolder$, Optional sMask$ = "*.*") As Collection

Private fso_ As Scripting.FileSystemObject

Public Function Setup()
  ' Mandatory setup function
  Set fso_ = New Scripting.FileSystemObject
  Call EnsureFolderExists(Dev_GetTestFolder)
End Function

Public Function Teardown()
  ' Mandatory teardown function
  Call fso_.DeleteFolder(Dev_GetTestFolder)
End Function

Public Function t_WindowsTempFolder()
  On Error GoTo PROPAGATE_ERROR
  
  Call Dev_ExpectEQ(Environ("TEMP"), WindowsTempFolder, "Temp folder")
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_Clipboard()
  On Error GoTo PROPAGATE_ERROR
  
  Dim sText$: sText = "test" & vbNewLine & "multiline"
  Call AddToClipboard(sText)
  Call Dev_ExpectEQ(sText, RetrieveFromClipboard, "Multiline text")
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_EnsureFolderExists()
  On Error GoTo PROPAGATE_ERROR
  
  On Error Resume Next
  Call EnsureFolderExists("")
  Call Dev_ExpectNoError("Empty input")
  On Error GoTo PROPAGATE_ERROR
  
  On Error Resume Next
  Call EnsureFolderExists("invalid input?")
  Call Dev_ExpectNoError("Invalid input")
  On Error GoTo PROPAGATE_ERROR
  
  Call Dev_NewCase("Single folder")
  Dim sFolder1$: sFolder1 = Dev_GetTestFolder & "\1"
  Call EnsureFolderExists(sFolder1)
  Call Dev_ExpectTrue(fso_.FolderExists(sFolder1))
  Call fso_.DeleteFolder(sFolder1)
  
  Call Dev_NewCase("Custom fso")
  Call EnsureFolderExists(sFolder1, fso_)
  Call Dev_ExpectTrue(fso_.FolderExists(sFolder1))
  Call fso_.DeleteFolder(sFolder1)
    
  Call Dev_NewCase("Chain folders")
  Dim sFolder2$: sFolder2 = sFolder1 & "\2"
  Call EnsureFolderExists(sFolder2)
  Call Dev_ExpectTrue(fso_.FolderExists(sFolder1))
  Call Dev_ExpectTrue(fso_.FolderExists(sFolder2))
  Call fso_.DeleteFolder(sFolder1)
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_CreateTextFileUTF16()
  On Error GoTo PROPAGATE_ERROR
  
  Call Dev_ExpectFalse(CreateTextFileUTF16(""), "Empty input")
  
  Call Dev_NewCase("Valid create file")
  Dim sFile$: sFile = Dev_GetTestFolder & "\" & "test.txt"
  Call Dev_ExpectTrue(CreateTextFileUTF16(sFile))
  Call Dev_ExpectTrue(fso_.FileExists(sFile), "Actually create the file")
  Dim fs As Scripting.TextStream: Set fs = fso_.OpenTextFile(sFile, ForAppending, Format:=TristateTrue)
  Call fs.WriteLine("test123")
  Call fs.Close
  
  Call Dev_NewCase("Do not overwrite")
  Call Dev_ExpectFalse(CreateTextFileUTF16(sFile, bOverwrite:=False))
  Set fs = fso_.OpenTextFile(sFile, ForReading, Format:=TristateTrue)
  Dim sText$: sText = fs.ReadLine
  Call fs.Close
  Call Dev_ExpectEQ("test123", sText, "Do not overwrite file")
  
  Call Dev_NewCase("Overwrite")
  Call Dev_ExpectTrue(CreateTextFileUTF16(sFile, bOverwrite:=True))
  Set fs = fso_.OpenTextFile(sFile, ForReading, Format:=TristateTrue)
  Call Dev_ExpectTrue(fs.AtEndOfStream, "Overwrite contents")
  Call fs.Close
  
  Call Dev_NewCase("Cannot overwrite")
  Set fs = fso_.OpenTextFile(sFile, ForWriting)
  Call Dev_ExpectFalse(CreateTextFileUTF16(sFile, bOverwrite:=True))
  Call fs.Close
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_AppendTextToFileUTF16()
  On Error GoTo PROPAGATE_ERROR
  
  Dim sFile$: sFile = Dev_GetTestFolder & "\" & "test.txt"
  Dim sMsg1$: sMsg1 = "testMsg1"
  Dim sMsg2$: sMsg2 = "testMsg2"
  
  Call Dev_ExpectFalse(AppendTextToFileUTF16("", ""), "Empty input")
  Call Dev_ExpectFalse(AppendTextToFileUTF16("", sMsg1), "Empty output file")
  Call Dev_ExpectFalse(AppendTextToFileUTF16(sFile, sMsg1), "Non-existent output file")
  
  Call Dev_NewCase("Append text to empty file")
  Call CreateTextFileUTF16(sFile)
  Call Dev_ExpectTrue(AppendTextToFileUTF16(sFile, sMsg1), "Append text")
  Dim fs As Scripting.TextStream: Set fs = fso_.OpenTextFile(sFile, ForReading, Format:=TristateTrue)
  Dim sText$: sText = fs.ReadAll
  Call fs.Close
  Call Dev_ExpectEQ(sMsg1 & vbNewLine, sText, "Confirm message")
  
  Call Dev_NewCase("Append text to file")
  Call Dev_ExpectTrue(AppendTextToFileUTF16(sFile, sMsg2), "Append text")
  Set fs = fso_.OpenTextFile(sFile, ForReading, Format:=TristateTrue)
  sText = fs.ReadAll
  Call fs.Close
  Call Dev_ExpectEQ(sMsg1 & vbNewLine & sMsg2 & vbNewLine, sText, "Confirm message")
  
  Call Dev_NewCase("Cannot append")
  Set fs = fso_.OpenTextFile(sFile, ForAppending, Format:=TristateTrue)
  Call Dev_ExpectFalse(AppendTextToFileUTF16(sFile, sMsg2))
  Call fs.Close

  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_ReadTextFromFileUTF16()
  On Error GoTo PROPAGATE_ERROR
  
  Dim sFile$: sFile = Dev_GetTestFolder & "\" & "test.txt"
  Dim sMsg$: sMsg = "testMsg"
  Dim sResult$: sResult = "test"
  
  Call Dev_NewCase("Empty input")
  Call Dev_ExpectFalse(ReadTextFromFileUTF16("", sResult))
  Call Dev_ExpectEQ("test", sResult, "Do not overwrite output on fail")
  
  Call Dev_NewCase("Non-existent output file")
  Call Dev_ExpectFalse(ReadTextFromFileUTF16(sFile, sResult))
  Call Dev_ExpectEQ("test", sResult, "Do not overwrite output on fail")
  
  Call CreateTextFileUTF16(sFile)
  Dim fs As Scripting.TextStream: Set fs = fso_.OpenTextFile(sFile, ForAppending, Format:=TristateTrue)
  Call fs.Write(sMsg)
  Call fs.Close
  
  Call Dev_NewCase("Valid read text")
  Call Dev_ExpectTrue(ReadTextFromFileUTF16(sFile, sResult))
  Call Dev_ExpectEQ(sMsg, sResult)

  Call Dev_NewCase("Read from occupied file")
  Set fs = fso_.OpenTextFile(sFile, ForAppending, Format:=TristateTrue)
  sResult = ""
  Call Dev_ExpectTrue(ReadTextFromFileUTF16(sFile, sResult))
  Call Dev_ExpectEQ(sMsg, sResult)
  Call fs.Close

  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_CopyFileOrFolder()
  On Error GoTo PROPAGATE_ERROR
  
  Call Dev_ExpectFalse(CopyFileOrFolder("", ""), "Empty input")
  
  Dim sFolder1$: sFolder1 = Dev_GetTestFolder & "\1"
  Dim sFolder2$: sFolder2 = Dev_GetTestFolder & "\2"
  Dim sFolder3$: sFolder3 = Dev_GetTestFolder & "\3"
  Dim sFile1$: sFile1 = Dev_GetTestFolder & "\" & "test1.txt"
  Dim sFile2$: sFile2 = Dev_GetTestFolder & "\" & "test2.txt"
  
  Call Dev_ExpectFalse(CopyFileOrFolder("", sFile1), "Empty source")
  
  Call fso_.CreateTextFile(sFile1).Close
  Call Dev_ExpectFalse(CopyFileOrFolder(sFile1, ""), "Empty destination")
  
  Call fso_.CreateFolder(sFolder1)
  Call Dev_ExpectFalse(CopyFileOrFolder(sFile1, sFolder1), "Copy file as folder")
  
  Call Dev_NewCase("Rename file")
  Call Dev_ExpectTrue(CopyFileOrFolder(sFile1, sFile2))
  Call Dev_ExpectTrue(fso_.FileExists(sFile1), "Do not delete initial file")
  Call Dev_ExpectTrue(fso_.FileExists(sFile2), "Create copy")
  Call fso_.DeleteFile(sFile2)
  
  Call Dev_NewCase("Custom fso")
  Call Dev_ExpectTrue(CopyFileOrFolder(sFile1, sFile2, fso_))
  Call Dev_ExpectTrue(fso_.FileExists(sFile1), "Do not delete initial file")
  Call Dev_ExpectTrue(fso_.FileExists(sFile2), "Create copy")
  
  Call Dev_NewCase("Replace existing file")
  Dim sMsg$: sMsg = "test1"
  Call AppendTextToFileUTF16(sFile1, sMsg)
  Call Dev_ExpectTrue(CopyFileOrFolder(sFile1, sFile2, fso_))
  Dim sResult$: Call ReadTextFromFileUTF16(sFile2, sResult)
  Call Dev_ExpectEQ(sMsg & vbNewLine, sResult)
  
  Call Dev_NewCase("Copy file to new folder")
  Dim sNewFile1$: sNewFile1 = sFolder1 & "\" & "test11.txt"
  Call Dev_ExpectTrue(CopyFileOrFolder(sFile1, sNewFile1))
  Call Dev_ExpectTrue(fso_.FileExists(sFile1), "Do not delete initial file")
  Call Dev_ExpectTrue(fso_.FileExists(sNewFile1), "Create copy")
  
  Call Dev_NewCase("Copy file to folder")
  Call fso_.CreateFolder(sFolder2)
  Dim sNewFile2$: sNewFile2 = sFolder2 & "\" & "test12.txt"
  Call Dev_ExpectTrue(CopyFileOrFolder(sFile1, sNewFile2))
  Call Dev_ExpectTrue(fso_.FileExists(sFile1), "Do not delete initial file")
  Call Dev_ExpectTrue(fso_.FileExists(sNewFile2), "Create copy")
  
  Call Dev_NewCase("Copy folder to new folder")
  Call Dev_ExpectTrue(CopyFileOrFolder(sFolder1, sFolder3))
  Call Dev_ExpectTrue(fso_.FolderExists(sFolder1), "Do not delete initial folder")
  Call Dev_ExpectTrue(fso_.FolderExists(sFolder3), "Create destination folder")
  Call Dev_ExpectTrue(fso_.FileExists(sFolder3 & "\" & "test11.txt"), "Copy contained file")
  
  Call Dev_NewCase("Copy folder to folder")
  Call Dev_ExpectTrue(CopyFileOrFolder(sFolder2, sFolder3))
  Call Dev_ExpectTrue(fso_.FolderExists(sFolder2), "Do not delete initial folder")
  Call Dev_ExpectTrue(fso_.FileExists(sFolder3 & "\" & "test11.txt"), "Keep existing contents")
  Call Dev_ExpectTrue(fso_.FileExists(sFolder3 & "\" & "test12.txt"), "Copy contained file")
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_MoveFileOrFolder()
  On Error GoTo PROPAGATE_ERROR
  
  Call Dev_ExpectFalse(MoveFileOrFolder("", ""), "Empty input")
  
  Dim sFolder1$: sFolder1 = Dev_GetTestFolder & "\1"
  Dim sFolder2$: sFolder2 = Dev_GetTestFolder & "\2"
  Dim sFolder3$: sFolder3 = Dev_GetTestFolder & "\3"
  Dim sFile1$: sFile1 = Dev_GetTestFolder & "\" & "test1.txt"
  Dim sFile2$: sFile2 = Dev_GetTestFolder & "\" & "test2.txt"
  
  Call Dev_ExpectFalse(MoveFileOrFolder("", sFile1), "Empty source")
  
  Call Dev_NewCase("Empty destination")
  Call CreateTextFileUTF16(sFile1)
  Call Dev_ExpectFalse(MoveFileOrFolder(sFile1, ""))
  Call Dev_ExpectTrue(fso_.FileExists(sFile1), "Do not delete source if move failed")
  
  Call Dev_NewCase("Move file as folder")
  Call fso_.CreateFolder(sFolder1)
  Call Dev_ExpectFalse(MoveFileOrFolder(sFile1, sFolder1))
  Call Dev_ExpectTrue(fso_.FileExists(sFile1), "Do not delete source if move failed")
  
  Call Dev_NewCase("Move self")
  Call Dev_ExpectFalse(MoveFileOrFolder(sFile1, sFile1))
  Call Dev_ExpectTrue(fso_.FileExists(sFile1), "Do not delete source if move failed")
  
  Call Dev_NewCase("Rename file")
  Call Dev_ExpectTrue(MoveFileOrFolder(sFile1, sFile2))
  Call Dev_ExpectFalse(fso_.FileExists(sFile1), "Delete initial file")
  Call Dev_ExpectTrue(fso_.FileExists(sFile2), "Renamed file exists")
  
  Call Dev_NewCase("Custom fso")
  Call Dev_ExpectTrue(MoveFileOrFolder(sFile2, sFile1, fso_))
  Call Dev_ExpectFalse(fso_.FileExists(sFile2), "Delete initial file")
  Call Dev_ExpectTrue(fso_.FileExists(sFile1), "Renamed file exists")
  
  Call Dev_NewCase("Replace existing file")
  Call CreateTextFileUTF16(sFile2)
  Dim sMsg$: sMsg = "test1"
  Call AppendTextToFileUTF16(sFile2, sMsg)
  Call Dev_ExpectTrue(MoveFileOrFolder(sFile2, sFile1, fso_))
  Call Dev_ExpectFalse(fso_.FileExists(sFile2), "Delete initial file")
  Call Dev_ExpectTrue(fso_.FileExists(sFile1), "Renamed file exists")
  Dim sResult$: Call ReadTextFromFileUTF16(sFile1, sResult)
  Call Dev_ExpectEQ(sMsg & vbNewLine, sResult, "Replace content")

  Call Dev_NewCase("Move file to new folder")
  Dim sNewFile1$: sNewFile1 = sFolder1 & "\" & "test11.txt"
  Call Dev_ExpectTrue(MoveFileOrFolder(sFile1, sNewFile1))
  Call Dev_ExpectFalse(fso_.FileExists(sFile1), "Delete initial file")
  Call Dev_ExpectTrue(fso_.FileExists(sNewFile1), "Moved file exists")

  Call Dev_NewCase("Move file to folder")
  Call fso_.CreateFolder(sFolder2)
  Dim sNewFile2$: sNewFile2 = sFolder2 & "\" & "test12.txt"
  Call Dev_ExpectTrue(MoveFileOrFolder(sNewFile1, sNewFile2))
  Call Dev_ExpectFalse(fso_.FileExists(sNewFile1), "Delete initial file")
  Call Dev_ExpectTrue(fso_.FileExists(sNewFile2), "Moved file exists")

  Call Dev_NewCase("Move folder to new folder")
  Call Dev_ExpectTrue(MoveFileOrFolder(sFolder2, sFolder3))
  Call Dev_ExpectFalse(fso_.FolderExists(sFolder2), "Delete initial folder")
  Call Dev_ExpectTrue(fso_.FolderExists(sFolder3), "Moved folder exists")
  Call Dev_ExpectTrue(fso_.FileExists(sFolder3 & "\" & "test12.txt"), "Move contained file")

  Call Dev_NewCase("Move folder to folder")
  Call CreateTextFileUTF16(sFile1)
  Call Dev_ExpectTrue(MoveFileOrFolder(sFolder3, sFolder1))
  Call Dev_ExpectFalse(fso_.FolderExists(sFolder3), "Delete initial folder")
  Call Dev_ExpectTrue(fso_.FileExists(sFile1), "Keep existing contents")
  Call Dev_ExpectTrue(fso_.FileExists(sFolder1 & "\" & "test12.txt"), "Move contained file")

  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

