VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "s_Path"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

Private path_ As API_Path

Public Function Setup()
  ' Mandatory setup function
  Set path_ = New API_Path
End Function

Public Function Teardown()
  ' Mandatory teardown function
End Function

Public Function t_FromString()
  On Error GoTo PROPAGATE_ERROR
  
  Call Dev_ExpectEQ("", path_.FromString("").Text, "Empty string")
  Call Dev_ExpectEQ("???.txt", path_.FromString("???.txt").Text, "Invalid string")
  Call Dev_ExpectEQ("D:\Dev\test.txt", path_.FromString("D:\Dev\test.txt").Text, "Global drive")
  Call Dev_ExpectEQ("..\local\test.txt", path_.FromString("..\local\test.txt").Text, "Local file")
  Call Dev_ExpectEQ("C:\LocalFolder", path_.FromString("C:\LocalFolder").Text, "Folder")
  Call Dev_ExpectEQ("%APPDATA%\text.txt", path_.FromString("%APPDATA%\text.txt").Text, "Environ")

  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_AccessProperties()
  On Error GoTo PROPAGATE_ERROR
  
  Call Dev_NewCase("Empty")
  Call path_.FromString("")
  Call Dev_ExpectEQ("", path_.ParentFolder, "ParentFolder")
  Call Dev_ExpectEQ("", path_.FileName, "FileName")
  Call Dev_ExpectEQ("", path_.BaseName, "BaseName")
  Call Dev_ExpectEQ("", path_.Extension, "Extension")
  
  Call Dev_NewCase("Invalid")
  Call path_.FromString("invalid?&.!")
  Call Dev_ExpectEQ("", path_.ParentFolder, "ParentFolder")
  Call Dev_ExpectEQ("invalid?&.!", path_.FileName, "FileName")
  Call Dev_ExpectEQ("invalid?&", path_.BaseName, "BaseName")
  Call Dev_ExpectEQ("!", path_.Extension, "Extension")
  
  Call Dev_NewCase("Valid file")
  Call path_.FromString("\\fs1.concept.ru\projects\test.txt")
  Call Dev_ExpectEQ("\\fs1.concept.ru\projects", path_.ParentFolder, "ParentFolder")
  Call Dev_ExpectEQ("test.txt", path_.FileName, "FileName")
  Call Dev_ExpectEQ("test", path_.BaseName, "BaseName")
  Call Dev_ExpectEQ("txt", path_.Extension, "Extension")
  
  Call Dev_NewCase("Valid folder")
  Call path_.FromString("C:\Windows\test")
  Call Dev_ExpectEQ("C:\Windows", path_.ParentFolder, "ParentFolder")
  Call Dev_ExpectEQ("test", path_.FileName, "FileName")
  Call Dev_ExpectEQ("test", path_.BaseName, "BaseName")
  Call Dev_ExpectEQ("", path_.Extension, "Extension")

  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_ToGlobal()
  On Error GoTo PROPAGATE_ERROR
  
  Call Dev_NewCase("Empty")
  Call path_.FromString("")
  Call Dev_ExpectEQ("", path_.ToGlobal("").Text)
  
  Call Dev_NewCase("Invalid")
  Call path_.FromString("invalid?&.!")
  Call Dev_ExpectEQ("\invalid?&.!", path_.ToGlobal("").Text)
  Call path_.FromString("invalid?&.!")
  Call Dev_ExpectEQ("\\fs1.concept.ru\invalid?&.!", path_.ToGlobal("\\fs1.concept.ru").Text)
  
  Call Dev_NewCase("Global to Global")
  Call path_.FromString("\\fs1.concept.ru\projects\test.txt")
  Call Dev_ExpectEQ("\\fs1.concept.ru\projects\test.txt", path_.ToGlobal("\\fs1.concept.ru").Text)
  
  Call Dev_NewCase("Environ to Global")
  Call path_.FromString("%APPDATA%\test.txt")
  Call Dev_ExpectEQ(VBA.Environ$("APPDATA") & "\test.txt", path_.ToGlobal("\\fs1.concept.ru").Text)
  Call path_.FromString("%INVALID_VARIABLE%\test.txt")
  Call Dev_ExpectEQ("\test.txt", path_.ToGlobal("\\fs1.concept.ru").Text)
  
  Call Dev_NewCase("Local to Global")
  Call path_.FromString("test.txt")
  Call Dev_ExpectEQ("\\fs1.concept.ru\test.txt", path_.ToGlobal("\\fs1.concept.ru").Text)
  Call path_.FromString("..\test.txt")
  Call Dev_ExpectEQ("\\fs1.concept.ru\1\2\test.txt", path_.ToGlobal("\\fs1.concept.ru\1\2\3").Text)
  Call path_.FromString("..\..\test.txt")
  Call Dev_ExpectEQ("\\fs1.concept.ru\1\test.txt", path_.ToGlobal("\\fs1.concept.ru\1\2\3").Text)
  Call Dev_ExpectEQ("\\fs1.concept.ru\1\test.txt", path_.Text)

  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_ToLocal()
  On Error GoTo PROPAGATE_ERROR
  
  Call Dev_NewCase("Empty")
  Call path_.FromString("")
  Call Dev_ExpectEQ("", path_.ToLocal("").Text)
  
  Call Dev_NewCase("Invalid")
  Call path_.FromString("invalid?&.!")
  Call Dev_ExpectEQ("invalid?&.!", path_.ToLocal("").Text)
  Call path_.FromString("invalid?&.!")
  Call Dev_ExpectEQ("invalid?&.!", path_.ToLocal("\\fs1.concept.ru").Text)
  
  Call Dev_NewCase("Global to Local")
  Call path_.FromString("\\fs1.concept.ru\projects\test.txt")
  Call Dev_ExpectEQ("test.txt", path_.Clone().ToLocal("\\fs1.concept.ru\projects").Text)
  Call Dev_ExpectEQ("..\test.txt", path_.Clone().ToLocal("\\fs1.concept.ru\projects\1").Text)
  Call Dev_ExpectEQ("projects\test.txt", path_.Clone().ToLocal("\\fs1.concept.ru").Text)
  Call Dev_ExpectEQ("..\..\projects\test.txt", path_.Clone().ToLocal("\\fs1.concept.ru\1\2").Text)

  Call Dev_NewCase("Environ to Local")
  Call path_.FromString("%APPDATA%\test.txt")
  Call Dev_ExpectEQ("%APPDATA%\test.txt", path_.ToLocal("\\fs1.concept.ru").Text)

  Call Dev_NewCase("Local to Local")
  Call path_.FromString("test.txt")
  Call Dev_ExpectEQ("test.txt", path_.ToLocal("\\fs1.concept.ru").Text)
  Call path_.FromString("..\test.txt")
  Call Dev_ExpectEQ("..\test.txt", path_.ToLocal("\\fs1.concept.ru\1\2\3").Text)
  Call path_.FromString("..\..\test.txt")
  Call Dev_ExpectEQ("..\..\test.txt", path_.ToLocal("\\fs1.concept.ru\1\2\3").Text)

  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_ToServer()
  On Error GoTo PROPAGATE_ERROR
  
  Call Dev_NewCase("Empty")
  Call path_.FromString("")
  Call Dev_ExpectEQ("", path_.ToServer("").Text)
  Call Dev_ExpectEQ("", path_.GlobalToServer().Text)
  
  Call Dev_NewCase("Invalid")
  Call path_.FromString("invalid?&.!")
  Call Dev_ExpectEQ("\invalid?&.!", path_.ToServer("").Text)
  Call Dev_ExpectEQ("\invalid?&.!", path_.GlobalToServer().Text)
  Call path_.FromString("invalid?&.!")
  Call Dev_ExpectEQ("\\fs1.concept.ru\invalid?&.!", path_.ToServer("\\fs1.concept.ru").Text)
  
  Call Dev_NewCase("Global to Server")
  Call path_.FromString("C:\Windows\test.txt")
  Call Dev_ExpectEQ("C:\Windows\test.txt", path_.Clone().ToServer("").Text)
  Call Dev_ExpectEQ("C:\Windows\test.txt", path_.Clone().GlobalToServer().Text)
  Call path_.FromString("P:\test\test.txt")
  Call Dev_ExpectEQ("\\fs1.concept.ru\projects\test\test.txt", path_.Clone().ToServer("").Text)
  Call Dev_ExpectEQ("\\fs1.concept.ru\projects\test\test.txt", path_.Clone().GlobalToServer().Text)

  Call Dev_NewCase("Environ to Server")
  Call path_.FromString("%APPDATA%\test.txt")
  Call Dev_ExpectEQ(path_.Clone.ToGlobal("").Text, path_.ToServer("").Text)

  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_CheckExistance()
  On Error GoTo PROPAGATE_ERROR
  
  Call Dev_NewCase("Empty")
  Call path_.FromString("")
  Call Dev_ExpectFalse(path_.LocalExists(""))
  Call Dev_ExpectFalse(path_.GlobalExists())
  
  Call Dev_NewCase("Invalid")
  Call path_.FromString("invalid?&.!")
  Call Dev_ExpectFalse(path_.LocalExists(""))
  Call Dev_ExpectFalse(path_.LocalExists("\\fs1.concept.ru"))
  Call Dev_ExpectFalse(path_.GlobalExists())
  
  Call Dev_NewCase("Valid file")
  Call path_.FromString(ThisWorkbook.FullName)
  Call Dev_ExpectTrue(path_.LocalExists(""))
  Call Dev_ExpectTrue(path_.LocalExists("\\fs1.concept.ru"))
  Call Dev_ExpectTrue(path_.GlobalExists())

  Call Dev_NewCase("Valid folder")
  Call path_.FromString(path_.ParentFolder)
  Call Dev_ExpectTrue(path_.LocalExists(""))
  Call Dev_ExpectTrue(path_.LocalExists("\\fs1.concept.ru"))
  Call Dev_ExpectTrue(path_.GlobalExists())

  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function
