VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "s_VsoUtilities"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
' ===== Test suite for Visio Utilities =======
Option Explicit

' Public Function VsoIncrementWidth(target As Visio.Shape, valueInMM&)

Private page_ As Visio.Page

Public Function Setup()
  ' Mandatory setup function
  Set page_ = ThisDocument.Pages(1)
  ThisDocument.DiagramServicesEnabled = visServiceStructureFull
End Function

Public Function Teardown()
  ' Mandatory teardown function
  ThisDocument.DiagramServicesEnabled = 0
  Call ClearAll
End Function

Public Function t_FindMaster()
  On Error GoTo PROPAGATE_ERROR
  
  Call Dev_ExpectNothing(FindMaster(ThisDocument, "Invalid"), "Invalid master name")
  Call Dev_ExpectNothing(FindMaster(Nothing, MASTER_ELEMENT), "Invalid document")
  Call Dev_ExpectEQ(ThisDocument.Masters.Item(MASTER_ELEMENT), FindMaster(ThisDocument, MASTER_ELEMENT), "Valid master")
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_GetContainingShape()
  On Error GoTo PROPAGATE_ERROR
  
  Call Dev_NewCase("Element")
  Dim elem As Visio.Shape: Set elem = page_.Drop(FindMaster(ThisDocument, MASTER_ELEMENT), 10, 10)
  Dim iArrow As Visio.Shape: Set iArrow = page_.Drop(FindMaster(ThisDocument, MASTER_CONNECTOR), 30, 30)
  Call Dev_ExpectNothing(GetContainingShape(elem))
  Call Dev_ExpectNothing(GetContainingShape(iArrow))
  
  Call Dev_NewCase("Container")
  Dim iContainer As Visio.Shape: Set iContainer = page_.Drop(FindMaster(ThisDocument, MASTER_CONTAINER), 20, 20)
  Call Dev_ExpectNothing(GetContainingShape(iContainer))
  Call iContainer.ContainerProperties.AddMember(elem, visMemberAddExpandContainer)
  Call iContainer.ContainerProperties.AddMember(iArrow, visMemberAddExpandContainer)
  Call Dev_ExpectNothing(GetContainingShape(iContainer))
  Call Dev_ExpectEQ(iContainer, GetContainingShape(elem))
  
  Call Dev_NewCase("List")
  Dim iList As Visio.Shape: Set iList = page_.Drop(FindMaster(ThisDocument, MASTER_LIST), 0, 0)
  Call iList.ContainerProperties.AddMember(iContainer, visMemberAddExpandContainer)
  Call Dev_ExpectNothing(GetContainingShape(iList))
  Call Dev_ExpectEQ(iContainer, GetContainingShape(elem))
  Call Dev_ExpectEQ(iContainer, GetContainingShape(iArrow))
  Call Dev_ExpectEQ(iList, GetContainingShape(iContainer))
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_VsoShapeExists()
  On Error GoTo PROPAGATE_ERROR
  
  Call Dev_NewCase("Empty page")
  Call Dev_ExpectFalse(VsoShapeExists(vbNullString, page_))
  Call Dev_ExpectFalse(VsoShapeExists("", page_))
  Call Dev_ExpectFalse(VsoShapeExists("invalid", page_))
  
  Call Dev_NewCase("Valid shape")
  Dim elem As Visio.Shape: Set elem = page_.Drop(FindMaster(ThisDocument, MASTER_ELEMENT), 10, 10)
  Call Dev_ExpectFalse(VsoShapeExists("", page_), "Empty input")
  Call Dev_ExpectFalse(VsoShapeExists("Test", page_), "Invalid name")
  Call Dev_ExpectTrue(VsoShapeExists(elem.Name, page_), "Default name")
  elem.Name = "Test1337"
  Call Dev_ExpectTrue(VsoShapeExists(elem.Name, page_), "Custom name")
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_VsoCalculateFillingFor()
  On Error GoTo PROPAGATE_ERROR
  
  Call Dev_ExpectEQ(0, VsoCalculateFillingFor(page_), "Empty page")
  
  Call Dev_NewCase("Arrow")
  Dim iArrow As Visio.Shape: Set iArrow = page_.Drop(FindMaster(ThisDocument, MASTER_CONNECTOR), 0, 0)
  Call Dev_ExpectEQ(0, VsoCalculateFillingFor(page_))
  
  Call Dev_NewCase("Valid shape")
  Dim elem As Visio.Shape: Set elem = page_.Drop(FindMaster(ThisDocument, MASTER_ELEMENT), 0, 0)
  Call Dev_ExpectNE(0, VsoCalculateFillingFor(page_))
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_VsoIsListContainer()
  On Error GoTo PROPAGATE_ERROR
  
  Dim testShape As Visio.Shape
  
  Call Dev_NewCase("Element")
  Set testShape = page_.Drop(FindMaster(ThisDocument, MASTER_ELEMENT), 0, 0)
  Call Dev_ExpectFalse(VsoIsListContainer(testShape))
  Call testShape.Delete
  
  Call Dev_NewCase("Arrow")
  Set testShape = page_.Drop(FindMaster(ThisDocument, MASTER_CONNECTOR), 0, 0)
  Call Dev_ExpectFalse(VsoIsListContainer(testShape))
  Call testShape.Delete
  
  Call Dev_NewCase("Container")
  Set testShape = page_.Drop(FindMaster(ThisDocument, MASTER_CONTAINER), 0, 0)
  Call Dev_ExpectFalse(VsoIsListContainer(testShape))
  Call testShape.Delete
  
  Call Dev_NewCase("List")
  Set testShape = page_.Drop(FindMaster(ThisDocument, MASTER_LIST), 0, 0)
  Call Dev_ExpectTrue(VsoIsListContainer(testShape))
  Call testShape.Delete
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_VsoIsMovable()
  On Error GoTo PROPAGATE_ERROR
  
  Call Dev_NewCase("Element")
  Dim elem As Visio.Shape: Set elem = page_.Drop(FindMaster(ThisDocument, MASTER_ELEMENT), 10, 10)
  Dim iArrow As Visio.Shape: Set iArrow = page_.Drop(FindMaster(ThisDocument, MASTER_CONNECTOR), 30, 30)
  Call Dev_ExpectTrue(VsoIsMovable(elem))
  Call Dev_ExpectFalse(VsoIsMovable(iArrow))

  Call Dev_NewCase("Container")
  Dim iContainer As Visio.Shape: Set iContainer = page_.Drop(FindMaster(ThisDocument, MASTER_CONTAINER), 20, 20)
  Call Dev_ExpectFalse(VsoIsMovable(iContainer))
  Call iContainer.ContainerProperties.AddMember(elem, visMemberAddExpandContainer)
  Call Dev_ExpectTrue(VsoIsMovable(elem))
  
  Call Dev_NewCase("List")
  Set elem = page_.Drop(FindMaster(ThisDocument, MASTER_ELEMENT), 0, 0)
  Dim iList As Visio.Shape: Set iList = page_.Drop(FindMaster(ThisDocument, MASTER_LIST), 0, 0)
  Call iList.ContainerProperties.InsertListMember(elem, 1)
  Call Dev_ExpectTrue(VsoIsMovable(iList))
  Call Dev_ExpectFalse(VsoIsMovable(elem))
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_VsoIsConnected()
  On Error GoTo PROPAGATE_ERROR
  
  Dim elem1 As Visio.Shape: Set elem1 = page_.Drop(FindMaster(ThisDocument, MASTER_ELEMENT), 10, 10)
  Dim elem2 As Visio.Shape: Set elem2 = page_.Drop(FindMaster(ThisDocument, MASTER_ELEMENT), 20, 20)
  Dim iArrow As Visio.Shape: Set iArrow = page_.Drop(FindMaster(ThisDocument, MASTER_CONNECTOR), 30, 30)
  
  Call Dev_NewCase("Not connected")
  Call Dev_ExpectFalse(VsoIsConnected(elem1, elem1), "Self")
  Call Dev_ExpectFalse(VsoIsConnected(iArrow, elem1), "Connector")
  Call Dev_ExpectFalse(VsoIsConnected(elem2, iArrow), "Connector")
  
  Call Dev_NewCase("Valid connection")
  Call elem1.AutoConnect(elem2, visAutoConnectDirNone)
  Call Dev_ExpectTrue(VsoIsConnected(elem1, elem2), "Straight connection")
  Call Dev_ExpectFalse(VsoIsConnected(elem2, elem1), "Reverse connection")
  Call Dev_ExpectFalse(VsoIsConnected(elem1, elem1), "Self connection")
  
  Call Dev_NewCase("Self connection")
  Call iArrow.CellsU("BeginX").GlueTo(elem1.CellsU("PinX"))
  Call iArrow.CellsU("EndX").GlueTo(elem1.CellsU("PinX"))
  Call Dev_ExpectTrue(VsoIsConnected(elem1, elem1))
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_VsoGeometricSort()
  On Error GoTo PROPAGATE_ERROR
  
  Call Dev_NewCase("Empty input")
  Dim ids() As Long
  On Error Resume Next
  Call VsoGeometricSort(ids, page_)
  Call Dev_ExpectNoError
  On Error GoTo PROPAGATE_ERROR
  
  Call Dev_NewCase("Valid elementary")
  Dim testIDs() As Long: ReDim testIDs(1 To 4)
  ReDim ids(1 To 4)
  Dim iShape1 As Visio.Shape: Set iShape1 = page_.Drop(FindMaster(ThisDocument, MASTER_ELEMENT), 0, 0)
  Dim iShape2 As Visio.Shape: Set iShape2 = page_.Drop(FindMaster(ThisDocument, MASTER_ELEMENT), 0, 1)
  Dim iShape3 As Visio.Shape: Set iShape3 = page_.Drop(FindMaster(ThisDocument, MASTER_ELEMENT), 1, 0)
  Dim iShape4 As Visio.Shape: Set iShape4 = page_.Drop(FindMaster(ThisDocument, MASTER_ELEMENT), 1, 1)
  ids(1) = iShape1.ID: testIDs(1) = iShape2.ID
  ids(2) = iShape2.ID: testIDs(2) = iShape4.ID
  ids(3) = iShape3.ID: testIDs(3) = iShape1.ID
  ids(4) = iShape4.ID: testIDs(4) = iShape3.ID
  Call VsoGeometricSort(ids, page_)
  Call Dev_ExpectEQ(testIDs, ids)
  
  Call Dev_NewCase("Valid different shapes")
  Set iShape1 = page_.DrawRectangle(0, 0, 10, 10)
  Set iShape2 = page_.DrawRectangle(11, 5, 17, -5)
  Set iShape3 = page_.DrawRectangle(0, 0, 10, 10 + 0.001)
  Set iShape4 = page_.DrawRectangle(0, 6, 7, 16)
  
  ids(1) = iShape1.ID: testIDs(1) = iShape4.ID
  ids(2) = iShape2.ID: testIDs(2) = iShape1.ID
  ids(3) = iShape3.ID: testIDs(3) = iShape3.ID
  ids(4) = iShape4.ID: testIDs(4) = iShape2.ID
  
  Call VsoGeometricSort(ids, page_)
  Call Dev_ExpectEQ(testIDs, ids)
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function
