VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "s_WordWrapper"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

' TODO: test reset paused status
' TODO: test reporter

Private fso_ As Scripting.FileSystemObject
Private word_ As API_WordWrapper

Public Function Setup()
  ' Mandatory setup function
  Set word_ = New API_WordWrapper
  Call word_.DisableMessages
  Call word_.CreateApplication
  Set fso_ = New Scripting.FileSystemObject
  Call EnsureFolderExists(Dev_GetTestFolder)
End Function

Public Function Teardown()
  ' Mandatory teardown function
  Call fso_.DeleteFolder(Dev_GetTestFolder)
  Call word_.ReleaseApplication
  Set word_ = Nothing
End Function

Public Function t_CreateApplication()
  On Error GoTo PROPAGATE_ERROR
  
  Dim word1 As New API_WordWrapper
  Dim word2 As New API_WordWrapper

  Call Dev_NewCase("Create visible application")
  Call Dev_AssertNotNothing(word1.CreateApplication(bIsVisible:=True))
  Call Dev_ExpectTrue(word1.Application.Visible)

  Call Dev_NewCase("Create hidden application")
  Call Dev_AssertNotNothing(word2.CreateApplication(bIsVisible:=False))
  Call Dev_ExpectFalse(word2.Application.Visible)
  
  Call Dev_NewCase("Create application twice")
  Dim app1 As Word.Application: Set app1 = word1.Application
  Call Dev_ExpectNotNothing(app1, "Create 1st application")
  Dim app2 As Word.Application: Set app2 = word1.CreateApplication
  Call Dev_ExpectNotNothing(app2, "Create 2nd application")
  Call Dev_ExpectObjectValid(app1, "Do not close first application before creating new")
  
  Call word1.ReleaseApplication
  Call word2.ReleaseApplication
  Call app1.Quit

  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_SetApplication()
  On Error GoTo PROPAGATE_ERROR
  
  Dim word1 As New API_WordWrapper
  Dim word2 As New API_WordWrapper
  Dim app1 As Word.Application: Set app1 = word1.CreateApplication(bIsVisible:=True)
  
  Call Dev_NewCase("Set application valid")
  Call word2.SetApplication(app1)
  Call Dev_ExpectEQ(app1, word2.Application)

  Call Dev_NewCase("Set application nothing")
  Call word2.SetApplication(Nothing)
  Call Dev_ExpectNothing(word2.Application)
  
  Call Dev_NewCase("Set self")
  Call word1.SetApplication(word1.Application)
  Call word1.ReleaseApplication
  Call Dev_ExpectObjectInvalid(app1, "Do not reset ownership on self set")
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_Destructor()
  On Error GoTo PROPAGATE_ERROR
  
  Dim word1 As New API_WordWrapper
  
  Call Dev_NewCase("No application")
  On Error Resume Next
  Set word1 = Nothing
  Call Dev_ExpectNoError
  On Error GoTo PROPAGATE_ERROR
  
  Call Dev_NewCase("Owned empty application")
  Dim app1 As Word.Application: Set app1 = word1.CreateApplication
  Set word1 = Nothing
  Call Sleep(500)
  Call Dev_ExpectObjectInvalid(app1)
  
  Call Dev_NewCase("Owned application with document")
  Set word1 = New API_WordWrapper
  Set app1 = word1.CreateApplication
  Dim doc1 As Word.Document: Set doc1 = word1.NewDocument
  Set word1 = Nothing
  Call Dev_ExpectObjectValid(doc1, "Do not close document")
  Call Dev_ExpectObjectValid(app1, "Do not close application")
  
  Call app1.Quit(SaveChanges:=False)
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_ReleaseApplication()
  On Error GoTo PROPAGATE_ERROR
  
  Dim word1 As New API_WordWrapper
  Dim word2 As New API_WordWrapper
  
  Call Dev_NewCase("No application")
  On Error Resume Next
  Call word1.ReleaseApplication
  Call Dev_ExpectNoError
  On Error GoTo PROPAGATE_ERROR
  
  Call Dev_NewCase("Owned application")
  Dim app1 As Word.Application: Set app1 = word1.CreateApplication
  Call word1.ReleaseApplication
  Call Dev_ExpectNothing(word1.Application, "Reset application")
  Call Dev_ExpectObjectInvalid(app1, "Close application")
  
  Call Dev_NewCase("Non-owned application")
  Set app1 = word1.CreateApplication
  Call word2.SetApplication(app1)
  Call word2.ReleaseApplication
  Call Dev_ExpectNothing(word2.Application, "Reset application")
  Call Dev_ExpectObjectValid(app1, "Do not close application")
  Call word1.ReleaseApplication
  
  Call Dev_NewCase("Owned application with owned open document")
  Set app1 = word1.CreateApplication
  Dim doc1 As Word.Document: Set doc1 = word1.NewDocument
  Call word1.ReleaseApplication
  Call Dev_ExpectNothing(word1.Application, "Reset application")
  Call Dev_ExpectObjectInvalid(doc1, "Close document")
  Call Dev_ExpectObjectInvalid(app1, "Close application")
  
  Call Dev_NewCase("Owned application with non-owned open document")
  Set app1 = word1.CreateApplication
  Call word2.SetApplication(app1)
  Set doc1 = word2.NewDocument
  Call word1.ReleaseApplication
  Call Dev_ExpectNothing(word1.Application, "Reset application")
  Call Dev_ExpectNotNothing(word2.Application, "Do not reset shared application")
  Call Dev_ExpectObjectInvalid(doc1, "Close document")
  Call Dev_ExpectObjectInvalid(app1, "Close application")
  
  On Error Resume Next
  Set word2 = Nothing
  Call Dev_ExpectNoError("Terminate correctly invalid application")
  On Error GoTo PROPAGATE_ERROR

  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_NewDocument()
  On Error GoTo PROPAGATE_ERROR

  Call Dev_NewCase("Default template")
  Call Dev_ExpectNotNothing(word_.NewDocument)
  Call word_.ReleaseDocument(bCloseApplication:=False)

  Call Dev_NewCase("Custom template")
  Dim sTemplate$: sTemplate = Dev_GetArtifactFolder & "\" & TEST_WORD_TEMPLATE
  Call Dev_AssertNotNothing(word_.NewDocument(sTemplate))
  Call Dev_ExpectEQ("test123", word_.Document.Words.First.Text)
  Call word_.ReleaseDocument(bCloseApplication:=False)
  
  Call Dev_NewCase("Missing template")
  Call Dev_ExpectNothing(word_.NewDocument(sTemplate:="invalid", bDefaultIfFail:=False), "No deafult fallback")
  Call Dev_AssertNotNothing(word_.NewDocument(sTemplate:="invalid", bDefaultIfFail:=True), "Default fallback")
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_SetDocument()
  On Error GoTo PROPAGATE_ERROR
  
  Dim word1 As New API_WordWrapper
  Dim word2 As New API_WordWrapper

  Call Dev_NewCase("Set nothing to nothing")
  On Error Resume Next
  Call word_.SetDocument(Nothing)
  Call Dev_ExpectNoError("No error")
  On Error GoTo PROPAGATE_ERROR
  Call Dev_ExpectNothing(word_.Document, "Document is set to nothing")
  Call Dev_ExpectNotNothing(word_.Application, "Application is not reset")
  
  Call Dev_NewCase("Set valid document")
  Call word1.SetApplication(word_.Application)
  Dim doc1 As Word.Document: Set doc1 = word1.NewDocument
  Call word_.SetDocument(doc1)
  Call Dev_ExpectEQ(doc1, word_.Document, "Check document")
  Call word_.SetDocument(Nothing)
  Call Dev_ExpectNothing(word_.Document, "Set valid to nothing")
  Call word1.ReleaseDocument(bCloseApplication:=False)
  
  Call Dev_NewCase("Set document from another application")
  Call word2.CreateApplication
  Call word1.SetApplication(word_.Application)
  Set doc1 = word2.NewDocument
  Call word1.SetDocument(doc1)
  Call Dev_ExpectEQ(doc1, word1.Document, "Check document")
  Call Dev_ExpectEQ(word2.Application, word1.Application, "Update application document")
  Call word1.ReleaseDocument(bCloseApplication:=False)
  Call Dev_ExpectObjectValid(doc1, "Do not transfer ownership")
  Call word2.ReleaseApplication
  
  Call Dev_NewCase("Transfer document ownership")
  Call word1.SetApplication(word_.Application)
  Set doc1 = word1.NewDocument
  Call word_.SetDocument(doc1, bOwnership:=True)
  Call Dev_ExpectEQ(doc1, word_.Document, "Check document")
  Call word_.ReleaseDocument(bCloseApplication:=False)
  Call Dev_ExpectObjectInvalid(doc1, "Transfer ownership")
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_OpenDocumentRead()
  On Error GoTo PROPAGATE_ERROR
  
  Dim word1 As New API_WordWrapper: Call word1.DisableMessages
  Dim word2 As New API_WordWrapper: Call word2.DisableMessages
  Dim doc1 As Word.Document
  Dim doc2 As Word.Document
  Dim sFile$: sFile = Dev_GetArtifactFolder & "\" & TEST_WORD_DOCUMENT
  Dim sInvalid$: sInvalid = Dev_GetArtifactFolder & "\" & TEST_WORD_INVALID
  Dim sText$: sText = "TestWord"

  Call Dev_NewCase("Invalid filename")
  Call Dev_ExpectNothing(word1.OpenDocument("invalid", bReadOnly:=True), "Wrapper without application")
  Call Dev_ExpectNothing(word1.Application, "Do not open application for invalid document")
  Call Dev_ExpectNothing(word_.OpenDocument("invalid", bReadOnly:=True), "Wrapper with application")
  Call Dev_ExpectObjectValid(word_.Application, "Do not reset application after failed document open")
  
  Call Dev_NewCase("Invalid file format")
  Call Dev_ExpectNothing(word1.OpenDocument(sInvalid, bReadOnly:=True), "Wrapper without application")
  Call Dev_ExpectNothing(word1.Application, "Do not open application for invalid document")
  Call Dev_ExpectNothing(word_.OpenDocument(sInvalid, bReadOnly:=True), "Wrapper with application")
  Call Dev_ExpectObjectValid(word_.Application, "Do not reset application after failed document open")
  
  Call Dev_NewCase("Valid file - existing application")
  Call Dev_AssertNotNothing(word_.OpenDocument(sFile, bReadOnly:=True))
  Call Dev_ExpectEQ(sText, word_.Document.Words.First.Text)
  Call Dev_ExpectNothing(word_.OpenDocument(sFile, bReadOnly:=True), "Cannot open another document while one is open already")
  Call word_.ReleaseDocument(bCloseApplication:=False)
  
  Call Dev_NewCase("Valid file - borrowing application")
  Call Dev_AssertNotNothing(word1.OpenDocument(sFile, bReadOnly:=True))
  Call Dev_ExpectEQ(sText, word1.Document.Words.First.Text)
  Call Dev_ExpectEQ(word_.Application, word1.Application, "Already open application should be used")
  Call word1.ReleaseDocument
  Call word1.ReleaseApplication
  Call Dev_ExpectObjectValid(word_.Application, "Do not claim ownership of borrowed application")
  
  Call Dev_NewCase("Valid file - no application")
  Call word_.ReleaseApplication
  Call Dev_AssertNotNothing(word_.OpenDocument(sFile, bReadOnly:=True))
  Call Dev_ExpectObjectValid(word_.Application, "Valid application")
  Call word_.ReleaseDocument(bCloseApplication:=False)
  
  Call Dev_NewCase("Valid file - already open READONLY in same application")
  Set doc1 = word_.OpenDocument(sFile, bReadOnly:=True)
  Call word2.SetApplication(word_.Application)
  Set doc2 = word2.OpenDocument(sFile, bReadOnly:=True)
  Call Dev_AssertNotNothing(doc2, "Open file")
  Call Dev_ExpectEQ(sText, doc2.Words.First.Text)
  Call Dev_ExpectEQ(ObjPtr(doc1), ObjPtr(doc2), "Do not reopen same document")
  Call word2.ReleaseDocument
  Call Dev_ExpectObjectValid(doc1, "Do not claim ownership of borrowed document")
  Call word_.ReleaseDocument(bCloseApplication:=False)
  
  Call Dev_NewCase("Valid file - already open MODIFY in same application")
  Set doc1 = word_.OpenDocument(sFile, bReadOnly:=False)
  Call word2.SetApplication(word_.Application)
  Set doc2 = word2.OpenDocument(sFile, bReadOnly:=True)
  Call Dev_AssertNotNothing(doc2, "Open file")
  Call Dev_ExpectEQ(sText, doc2.Words.First.Text)
  Call Dev_ExpectEQ(ObjPtr(doc1), ObjPtr(doc2), "Do not reopen same document")
  Call word2.ReleaseDocument
  Call Dev_ExpectObjectValid(doc1, "Do not claim ownership of borrowed document")
  Call word_.ReleaseDocument(bCloseApplication:=False)
  
  Call Dev_NewCase("Valid file - already open in another application")
  Call word2.CreateApplication
  Set doc2 = word2.OpenDocument(sFile, bReadOnly:=True)
  Call Dev_ExpectEQ(sText, doc2.Words.First.Text)
  Call Dev_ExpectNE(ObjPtr(doc1), ObjPtr(doc2), "Reopen document in new application")
  Call word2.ReleaseDocument
  Call word2.ReleaseApplication
  
  Call word_.ReleaseDocument

  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_OpenDocumentModify()
  On Error GoTo PROPAGATE_ERROR
  
  Dim word1 As New API_WordWrapper: Call word1.DisableMessages
  Dim word2 As New API_WordWrapper: Call word2.DisableMessages
  Dim doc1 As Word.Document
  Dim doc2 As Word.Document
  Dim sFile$: sFile = Dev_GetTestFolder & "\" & TEST_WORD_DOCUMENT
  Dim sText1$: sText1 = "TestWord"
  Dim sText2$: sText2 = "Test1Word2"
  
  Call fso_.CopyFile(Dev_GetArtifactFolder & "\" & TEST_WORD_DOCUMENT, sFile)

  Call Dev_NewCase("Valid file - existing application")
  Call Dev_AssertNotNothing(word_.OpenDocument(sFile, bReadOnly:=False))
  Call Dev_ExpectEQ(sText1, word_.Document.Words.First.Text, "Validate opened file")
  word_.Document.Words.First = sText2
  Call word_.ReleaseDocument(bCloseApplication:=False, bSaveChanges:=True)
  Call Dev_AssertNotNothing(word_.OpenDocument(sFile, bReadOnly:=True))
  Call Dev_ExpectEQ(sText2, word_.Document.Words.First.Text, "Confirm modification")
  Call word_.ReleaseDocument(bCloseApplication:=False)
  
  Call Dev_NewCase("Readonly file")
  fso_.GetFile(sFile).Attributes = ReadOnly
  Call Dev_ExpectNothing(word_.OpenDocument(sFile, bReadOnly:=False))
  Call Dev_ExpectNotNothing(word_.OpenDocument(sFile, bReadOnly:=True))
  Call word_.ReleaseDocument(bCloseApplication:=False)
  fso_.GetFile(sFile).Attributes = Normal
  
  Call Dev_NewCase("Valid file - already open MODIFY in same application")
  Call word_.OpenDocument(sFile, bReadOnly:=False)
  Call word2.SetApplication(word_.Application)
  Set doc2 = word2.OpenDocument(sFile, bReadOnly:=False)
  Call Dev_ExpectNotNothing(doc2, "Find already opened file")
  Call Dev_ExpectEQ(ObjPtr(word_.Document), ObjPtr(doc2), "Get same document")
  Call word2.ReleaseDocument(bCloseApplication:=False)
  Call word_.ReleaseDocument(bCloseApplication:=False)

  Call Dev_NewCase("Valid file - already open READONLY in same application")
  Call word_.OpenDocument(sFile, bReadOnly:=True)
  Call word2.SetApplication(word_.Application)
  Call Dev_ExpectNothing(word2.OpenDocument(sFile, bReadOnly:=False), "Do not open blocked document")
  Call word_.ReleaseDocument(bCloseApplication:=False)
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_ReleaseDocument()
  On Error GoTo PROPAGATE_ERROR
  
  Dim word1 As New API_WordWrapper: Call word1.DisableMessages
  Dim word2 As New API_WordWrapper: Call word2.DisableMessages
  Dim sFile$: sFile = Dev_GetTestFolder & "\" & TEST_WORD_DOCUMENT
  Dim doc1 As Word.Document
  Dim doc2 As Word.Document
  Dim app1 As Word.Application
  Dim sText$: sText = "TestWord"
  
  Set app1 = word_.Application
  
  Call Dev_NewCase("Empty wrapper")
  Call Dev_ExpectFalse(word1.ReleaseDocument)
    
  Call Dev_NewCase("No open document")
  Call Dev_ExpectFalse(word_.ReleaseDocument(bCloseApplication:=False), "Do not force close application")
  Call Dev_ExpectNotNothing(word_.Application, "Do not reset application")
  Call Dev_ExpectObjectValid(app1, "Do not close application")
  Call Dev_ExpectFalse(word_.ReleaseDocument(bCloseApplication:=True), "Force close application")
  Call Dev_ExpectNotNothing(word_.Application, "Do not reset application")
  Call Dev_ExpectObjectValid(app1, "Do not close application")
  
  Call Dev_NewCase("Release owned document - newly created no save changes")
  Set doc1 = word_.NewDocument
  doc1.Words.First = sText
  Call Dev_ExpectTrue(word_.ReleaseDocument(bCloseApplication:=False, bSaveChanges:=False))
  Call Dev_ExpectNotNothing(word_.Application, "Do not reset application")
  Call Dev_ExpectNothing(word_.Document, "Reset document")
  Call Dev_ExpectObjectValid(app1, "Do not close application")
  Call Dev_ExpectObjectInvalid(doc1, "Close document")
  
  Call Dev_NewCase("Release owned document - newly created save changes")
  Set doc1 = word_.NewDocument
  doc1.Words.First = sText
  Call Dev_ExpectTrue(word_.ReleaseDocument(bCloseApplication:=False, bSaveChanges:=True))
  Call Dev_ExpectNotNothing(word_.Application, "Do not reset application")
  Call Dev_ExpectNothing(word_.Document, "Reset document")
  Call Dev_ExpectObjectInvalid(doc1, "Close document")
  Call Dev_ExpectObjectValid(app1, "Do not close application")
  
  Call Dev_NewCase("Release owned document - dont save changes")
  Set doc1 = word_.NewDocument
  Call word_.SaveAs(sFile)
  word_.Document.Words.First = sText
  Call Dev_ExpectTrue(word_.ReleaseDocument(bCloseApplication:=False, bSaveChanges:=False))
  Call Dev_ExpectNothing(word_.Document, "Reset document")
  Call Dev_ExpectNotNothing(word_.Application, "Do not reset application")
  Call Dev_ExpectObjectInvalid(doc1, "Close document")
  Call Dev_ExpectObjectValid(app1, "Do not close application")
  Call word_.OpenDocument(sFile, bReadOnly:=False)
  Call Dev_ExpectNE(sText, word_.Document.Words.First, "Do not save changes")
  Call word_.ReleaseDocument(bCloseApplication:=False)
  
  Call Dev_NewCase("Release owned document - save changes")
  Set doc1 = word_.NewDocument
  Call word_.SaveAs(sFile)
  word_.Document.Words.First = sText
  Call Dev_ExpectTrue(word_.ReleaseDocument(bCloseApplication:=False, bSaveChanges:=True))
  Call Dev_ExpectNothing(word_.Document, "Reset document")
  Call Dev_ExpectNotNothing(word_.Application, "Do not reset application")
  Call Dev_ExpectObjectInvalid(doc1, "Close document")
  Call Dev_ExpectObjectValid(app1, "Do not close application")
  Call word_.OpenDocument(sFile, bReadOnly:=False)
  Call Dev_ExpectEQ(sText, word_.Document.Words.First, "Save changes")
  Call word_.ReleaseDocument(bCloseApplication:=False)
  
  Call Dev_NewCase("Release owned document - close app")
  Set doc1 = word_.NewDocument
  Call Dev_ExpectTrue(word_.ReleaseDocument(bCloseApplication:=True, bSaveChanges:=True))
  Call Dev_ExpectNothing(word_.Document, "Reset document")
  Call Dev_ExpectNothing(word_.Application, "Reset application")
  Call Dev_ExpectObjectInvalid(doc1, "Close document")
  Call Dev_ExpectObjectInvalid(app1, "Close application")
  
  Call Dev_NewCase("Release non-owned document")
  Set doc1 = word1.NewDocument
  Call word2.SetDocument(doc1, bOwnership:=False)
  Call Dev_ExpectTrue(word2.ReleaseDocument)
  Call Dev_ExpectNothing(word2.Document, "Reset document")
  Call Dev_ExpectNotNothing(word1.Document, "Do not reset owner")
  Call Dev_ExpectObjectValid(doc1, "Do not close document")
  Call word1.ReleaseDocument
  
  Call Dev_NewCase("Release owned dangling document")
  Set doc1 = word1.NewDocument
  Call word2.SetDocument(doc1, bOwnership:=True)
  Call word1.ReleaseDocument
  Call Dev_ExpectObjectInvalid(doc1, "Close document")
  Call Dev_ExpectFalse(word2.ReleaseDocument)
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_PauseUI()
  On Error GoTo PROPAGATE_ERROR
  
  Dim word1 As New API_WordWrapper
  Dim doc As Word.Document
  
  Call Dev_NewCase("Empty wrapper")
  On Error Resume Next
  Call word1.PauseUI
  Call Dev_ExpectNoError
  Call Dev_ExpectFalse(word1.IsUIPaused, "Pause")
  On Error Resume Next
  Call word1.ResumeUI
  Call Dev_ExpectNoError
  Call Dev_ExpectFalse(word1.IsUIPaused, "Resume")
  On Error GoTo PROPAGATE_ERROR
  
  Call Dev_NewCase("No document")
  On Error Resume Next
  Call word_.PauseUI
  Call Dev_ExpectNoError
  Call Dev_ExpectFalse(word_.IsUIPaused)
  On Error Resume Next
  Call word_.ResumeUI
  Call Dev_ExpectNoError
  Call Dev_ExpectFalse(word_.IsUIPaused)
  On Error GoTo PROPAGATE_ERROR
  
  Call Dev_NewCase("Owned document")
  Set doc = word_.NewDocument
  Call Dev_ExpectTrue(word_.Application.ScreenUpdating)
  Call word_.PauseUI
  Call Dev_ExpectTrue(word_.IsUIPaused, "Pause")
  Call Dev_ExpectFalse(word_.Application.ScreenUpdating, "Pause")
  Call word_.ResumeUI
  Call Dev_ExpectFalse(word_.IsUIPaused, "Resume")
  Call Dev_ExpectTrue(word_.Application.ScreenUpdating, "Resume")
  Call word_.ReleaseDocument
  
  Call Dev_NewCase("Non-owned document")
  Set doc = word1.NewDocument
  Call word_.SetDocument(doc, bOwnership:=False)
  Call word_.PauseUI
  Call Dev_ExpectTrue(word_.IsUIPaused, "Pause")
  Call Dev_ExpectFalse(word1.IsUIPaused, "Do not pause owner")
  Call Dev_ExpectFalse(word_.Application.ScreenUpdating, "Pause")
  Call word_.ResumeUI
  Call Dev_ExpectFalse(word_.IsUIPaused, "Resume")
  Call Dev_ExpectTrue(word_.Application.ScreenUpdating, "Resume")
  
  Call Dev_NewCase("Overlapped pause different wrappers")
  Call word_.PauseUI
  Call word1.PauseUI
  Call Dev_ExpectTrue(word_.IsUIPaused, "Pause")
  Call Dev_ExpectTrue(word1.IsUIPaused, "Pause")
  Call Dev_ExpectFalse(word_.Application.ScreenUpdating, "Pause")
  Call word1.ResumeUI
  Call Dev_ExpectFalse(word1.IsUIPaused, "Resume inner")
  Call Dev_ExpectTrue(word_.IsUIPaused, "Do not resume outer")
  Call Dev_ExpectFalse(word_.Application.ScreenUpdating, "Do not enable UI")
  Call word_.ResumeUI
  Call Dev_ExpectFalse(word1.IsUIPaused, "Resume inner")
  Call Dev_ExpectFalse(word_.IsUIPaused, "Resume outer")
  Call Dev_ExpectTrue(word_.Application.ScreenUpdating, "Enable UI")
  
  Call word1.ReleaseDocument
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_SaveAs()
  On Error GoTo PROPAGATE_ERROR
  
  Dim sFile$: sFile = Dev_GetTestFolder & "\" & TEST_WORD_DOCUMENT
  Dim sTemplate$: sTemplate = Dev_GetTestFolder & "\" & "testFile"
  Dim sText$: sText = "TestWord"
  Dim sTarget$
  Call fso_.CopyFile(Dev_GetArtifactFolder & "\" & TEST_WORD_DOCUMENT, sFile)
  Dim oReader As New API_WordWrapper
  Call oReader.CreateApplication
  Call oReader.DisableMessages
  
  Call Dev_NewCase("No document")
  On Error Resume Next
  Call Dev_ExpectFalse(word_.SaveAs(sTemplate & ".docx"))
  Call Dev_ExpectNoError
  On Error Resume Next
  
  Call Dev_AssertTrue(word_.OpenDocument(sFile))
  Call Dev_ExpectFalse(word_.SaveAs("invalid!?"), "Invalid filename")
  
  Call Dev_NewCase("Save as docx")
  sTarget = sTemplate & ".docx"
  Call Dev_ExpectTrue(word_.SaveAs(sTarget))
  Call Dev_AssertTrue(fso_.FileExists(sTarget), "File created")
  Call Dev_ExpectTrue(oReader.OpenDocument(sTarget, bReadOnly:=True), "Open saved document")
  Call Dev_ExpectEQ(sText, oReader.Document.Words.First, "Validate contents")
  Call oReader.ReleaseDocument(bCloseApplication:=False)
  
  Call Dev_NewCase("Save as docm")
  sTarget = sTemplate & ".docm"
  Call Dev_ExpectTrue(word_.SaveAs(sTarget))
  Call Dev_AssertTrue(fso_.FileExists(sTarget), "File created")
  Call Dev_ExpectTrue(oReader.OpenDocument(sTarget, bReadOnly:=True), "Open saved document")
  Call Dev_ExpectEQ(sText, oReader.Document.Words.First, "Validate contents")
  Call oReader.ReleaseDocument(bCloseApplication:=False)
  
  Call Dev_NewCase("Save as dotx")
  sTarget = sTemplate & ".dotx"
  Call Dev_ExpectTrue(word_.SaveAs(sTarget))
  Call Dev_AssertTrue(fso_.FileExists(sTarget), "File created")
  Call Dev_ExpectTrue(oReader.OpenDocument(sTarget, bReadOnly:=True), "Open saved document")
  Call Dev_ExpectEQ(sText, oReader.Document.Words.First, "Validate contents")
  Call oReader.ReleaseDocument(bCloseApplication:=False)
  
  Call Dev_NewCase("Save as dotm")
  sTarget = sTemplate & ".dotm"
  Call Dev_ExpectTrue(word_.SaveAs(sTarget))
  Call Dev_AssertTrue(fso_.FileExists(sTarget), "File created")
  Call Dev_ExpectTrue(oReader.OpenDocument(sTarget, bReadOnly:=True), "Open saved document")
  Call Dev_ExpectEQ(sText, oReader.Document.Words.First, "Validate contents")
  Call oReader.ReleaseDocument(bCloseApplication:=False)
  
  Call Dev_NewCase("Overwrite")
  sTarget = sTemplate & ".docx"
  sText = "test2"
  word_.Document.Words.First = sText
  Call Dev_ExpectTrue(word_.SaveAs(sTarget))
  Call Dev_AssertTrue(fso_.FileExists(sTarget), "File created")
  Call Dev_ExpectTrue(oReader.OpenDocument(sTarget, bReadOnly:=True), "Open saved document")
  Call Dev_ExpectEQ(sText, oReader.Document.Words.First, "Validate contents")
  Call oReader.ReleaseDocument(bCloseApplication:=False)
  
  Call oReader.ReleaseApplication
  Call word_.ReleaseDocument
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function
